@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,6 @@ import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.index.Index;
 import org.hsqldb.lib.HsqlList;
-import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.lib.OrderedIntHashSet;
 import org.hsqldb.navigator.RowIterator;
 import org.hsqldb.persist.PersistentStore;
@@ -49,7 +48,7 @@ import org.hsqldb.types.Types;
 /**
  * @author Campbell Burnet (boucherb@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.2.9
+ * @version 2.3.4
  * @since 1.9.0
  */
 public class ExpressionLogical extends Expression {
@@ -142,7 +141,13 @@ public class ExpressionLogical extends Expression {
 
             // fall through
             case OpTypes.NOT_EQUAL :
-            case OpTypes.OVERLAPS :
+            case OpTypes.RANGE_CONTAINS :
+            case OpTypes.RANGE_EQUALS :
+            case OpTypes.RANGE_OVERLAPS :
+            case OpTypes.RANGE_IMMEDIATELY_PRECEDES :
+            case OpTypes.RANGE_IMMEDIATELY_SUCCEEDS :
+            case OpTypes.RANGE_PRECEDES :
+            case OpTypes.RANGE_SUCCEEDS :
             case OpTypes.NOT_DISTINCT :
             case OpTypes.IN :
             case OpTypes.MATCH_SIMPLE :
@@ -255,6 +260,8 @@ public class ExpressionLogical extends Expression {
                         isSingleColumnEqual = true;
                     }
                     break;
+
+                default :
             }
         } else if (nodes[RIGHT].opType == OpTypes.COLUMN) {
             nodes[RIGHT].nullability = SchemaObject.Nullability.NO_NULLS;
@@ -271,6 +278,8 @@ public class ExpressionLogical extends Expression {
                         isSingleColumnEqual = true;
                     }
                     break;
+
+                default :
             }
         }
     }
@@ -601,10 +610,26 @@ public class ExpressionLogical extends Expression {
                 sb.append(Tokens.T_EXISTS);
                 break;
 
-            case OpTypes.OVERLAPS :
+            case OpTypes.RANGE_CONTAINS :
+                sb.append(Tokens.T_CONTAINS);
+                break;
+
+            case OpTypes.RANGE_EQUALS :
+                sb.append(Tokens.T_EQUALS);
+                break;
+
+            case OpTypes.RANGE_OVERLAPS :
                 sb.append(Tokens.T_OVERLAPS);
                 break;
 
+            case OpTypes.RANGE_PRECEDES :
+                sb.append(Tokens.T_PRECEDES);
+                break;
+
+            case OpTypes.RANGE_SUCCEEDS :
+                sb.append(Tokens.T_SUCCEEDS);
+                break;
+
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500,
                                          "ExpressionLogical");
@@ -808,8 +833,14 @@ public class ExpressionLogical extends Expression {
 
                 break;
             }
-            case OpTypes.OVERLAPS :
-                resolveTypesForOverlaps();
+            case OpTypes.RANGE_CONTAINS :
+            case OpTypes.RANGE_EQUALS :
+            case OpTypes.RANGE_OVERLAPS :
+            case OpTypes.RANGE_IMMEDIATELY_PRECEDES :
+            case OpTypes.RANGE_IMMEDIATELY_SUCCEEDS :
+            case OpTypes.RANGE_PRECEDES :
+            case OpTypes.RANGE_SUCCEEDS :
+                resolveTypesForPeriodPredicates(session);
                 break;
 
             case OpTypes.IN :
@@ -954,6 +985,10 @@ public class ExpressionLogical extends Expression {
                             ^ nodes[RIGHT].dataType.isDateTimeTypeWithZone()) {
                         nodes[LEFT] = new ExpressionOp(nodes[LEFT]);
                     }
+                } else if (nodes[LEFT].dataType.canConvertFrom(
+                        nodes[RIGHT].dataType)) {
+                    nodes[RIGHT] = ExpressionOp.getCastExpression(session,
+                            nodes[RIGHT], nodes[LEFT].dataType);
                 } else {
                     throw Error.error(ErrorCode.X_42562);
                 }
@@ -1101,8 +1136,20 @@ public class ExpressionLogical extends Expression {
         return false;
     }
 
-    void resolveTypesForOverlaps() {
+    void resolveTypesForPeriodPredicates(Session session) {
 
+        // convert CONTAINS right part if necessary
+        if (nodes[RIGHT].nodes.length == 0) {
+            Expression[] newNodes = new Expression[] {
+                nodes[RIGHT], new ExpressionValue(null, nodes[RIGHT].dataType)
+            };
+
+            nodes[RIGHT] = new Expression(OpTypes.ROW, newNodes);
+
+            nodes[RIGHT].resolveTypes(session, null);
+        }
+
+        // end convert
         if (nodes[LEFT].nodes[0].isUnresolvedParam()) {
             nodes[LEFT].nodes[0].dataType = nodes[RIGHT].nodes[0].dataType;
         }
@@ -1124,16 +1171,13 @@ public class ExpressionLogical extends Expression {
             nodes[RIGHT].nodes[1].dataType = nodes[LEFT].nodes[0].dataType;
         }
 
-        if (!DTIType
-                .isValidDatetimeRange(nodes[LEFT].nodes[0]
-                    .dataType, nodes[LEFT].nodes[1].dataType) || !DTIType
-                        .isValidDatetimeRange(nodes[RIGHT].nodes[0]
-                            .dataType, nodes[RIGHT].nodes[1].dataType)) {
+        if (!DTIType.isValidDatetimeRange(nodes[LEFT].nodes[0].dataType,
+                                          nodes[LEFT].nodes[1].dataType)) {
             throw Error.error(ErrorCode.X_42563);
         }
 
-        if (!DTIType.isValidDatetimeRange(nodes[LEFT].nodes[0].dataType,
-                                          nodes[LEFT].nodes[1].dataType)) {
+        if (!DTIType.isValidDatetimeRange(nodes[RIGHT].nodes[0].dataType,
+                                          nodes[RIGHT].nodes[1].dataType)) {
             throw Error.error(ErrorCode.X_42563);
         }
 
@@ -1243,11 +1287,69 @@ public class ExpressionLogical extends Expression {
                 return nodes[LEFT].getValue(session) == null ? Boolean.TRUE
                                                              : Boolean.FALSE;
             }
-            case OpTypes.OVERLAPS : {
+            case OpTypes.RANGE_CONTAINS : {
+                Object[] left  = nodes[LEFT].getRowValue(session);
+                Object[] right = nodes[RIGHT].getRowValue(session);
+                boolean pointOfTime = nodes[RIGHT].exprSubType
+                                      != OpTypes.RANGE_EQUALS;
+
+                return DateTimeType.contains(session, left,
+                                             nodes[LEFT].nodeDataTypes, right,
+                                             nodes[RIGHT].nodeDataTypes,
+                                             pointOfTime);
+            }
+            case OpTypes.RANGE_EQUALS : {
                 Object[] left  = nodes[LEFT].getRowValue(session);
                 Object[] right = nodes[RIGHT].getRowValue(session);
 
-                return DateTimeType.overlaps(session, left,
+                return DateTimeType.equals(session, left,
+                                           nodes[LEFT].nodeDataTypes, right,
+                                           nodes[RIGHT].nodeDataTypes);
+            }
+            case OpTypes.RANGE_OVERLAPS : {
+                Object[] left  = nodes[LEFT].getRowValue(session);
+                Object[] right = nodes[RIGHT].getRowValue(session);
+
+                if (nodes[LEFT].exprSubType == OpTypes.RANGE_EQUALS) {
+                    return DateTimeType.overlaps(session, left,
+                                                 nodes[LEFT].nodeDataTypes,
+                                                 right,
+                                                 nodes[RIGHT].nodeDataTypes);
+                } else {
+                    return DateTimeType.overlapsRelaxed(
+                        session, left, nodes[LEFT].nodeDataTypes, right,
+                        nodes[RIGHT].nodeDataTypes);
+                }
+            }
+            case OpTypes.RANGE_PRECEDES : {
+                Object[] left  = nodes[LEFT].getRowValue(session);
+                Object[] right = nodes[RIGHT].getRowValue(session);
+
+                return DateTimeType.precedes(session, left,
+                                             nodes[LEFT].nodeDataTypes, right,
+                                             nodes[RIGHT].nodeDataTypes);
+            }
+            case OpTypes.RANGE_IMMEDIATELY_PRECEDES : {
+                Object[] left  = nodes[LEFT].getRowValue(session);
+                Object[] right = nodes[RIGHT].getRowValue(session);
+
+                return DateTimeType.immediatelyPrecedes(session, left,
+                        nodes[LEFT].nodeDataTypes, right,
+                        nodes[RIGHT].nodeDataTypes);
+            }
+            case OpTypes.RANGE_IMMEDIATELY_SUCCEEDS : {
+                Object[] left  = nodes[LEFT].getRowValue(session);
+                Object[] right = nodes[RIGHT].getRowValue(session);
+
+                return DateTimeType.immediatelySucceeds(session, left,
+                        nodes[LEFT].nodeDataTypes, right,
+                        nodes[RIGHT].nodeDataTypes);
+            }
+            case OpTypes.RANGE_SUCCEEDS : {
+                Object[] left  = nodes[LEFT].getRowValue(session);
+                Object[] right = nodes[RIGHT].getRowValue(session);
+
+                return DateTimeType.succeeds(session, left,
                                              nodes[LEFT].nodeDataTypes, right,
                                              nodes[RIGHT].nodeDataTypes);
             }
@@ -1337,7 +1439,7 @@ public class ExpressionLogical extends Expression {
 
                 if (nodes[LEFT].dataType != null
                         && nodes[LEFT].dataType.isArrayType()) {
-                    return compareValues(session, (Object) o1, (Object) o2);
+                    return compareValues(session, o1, o2);
                 }
 
                 if (o1 instanceof Object[]) {
@@ -1413,19 +1515,16 @@ public class ExpressionLogical extends Expression {
      * For MATCH SIMPLE and FULL expressions, nulls in left are handled
      * prior to calling this method
      */
-    private Boolean compareValues(Session session, Object[] left,
-                                  Object[] right) {
+    private Boolean compareValues(Session session, Object[] leftList,
+                                  Object[] rightList) {
 
         int     result  = 0;
         boolean hasNull = false;
 
-        if (left == null || right == null) {
+        if (leftList == null || rightList == null) {
             return null;
         }
 
-        Object[] leftList  = (Object[]) left;
-        Object[] rightList = (Object[]) right;
-
         for (int i = 0; i < nodes[LEFT].nodes.length; i++) {
             if (leftList[i] == null) {
                 if (opType == OpTypes.MATCH_PARTIAL
@@ -1625,6 +1724,9 @@ public class ExpressionLogical extends Expression {
                             }
 
                             hasMatch = true;
+                            break;
+
+                        default :
                     }
                 }
 
@@ -1721,7 +1823,7 @@ public class ExpressionLogical extends Expression {
 
     private Boolean testAllAnyCondition(Session session) {
 
-        Object[]     rowData = (Object[]) nodes[LEFT].getRowValue(session);
+        Object[]     rowData = nodes[LEFT].getRowValue(session);
         TableDerived td      = nodes[RIGHT].table;
 
         td.materialiseCorrelated(session);
@@ -2243,7 +2345,13 @@ public class ExpressionLogical extends Expression {
                 return nodes[LEFT].costFactor(session, rangeVar, opType)
                        + nodes[RIGHT].costFactor(session, rangeVar, opType);
             }
-            case OpTypes.OVERLAPS :
+            case OpTypes.RANGE_CONTAINS :
+            case OpTypes.RANGE_EQUALS :
+            case OpTypes.RANGE_OVERLAPS :
+            case OpTypes.RANGE_IMMEDIATELY_PRECEDES :
+            case OpTypes.RANGE_IMMEDIATELY_SUCCEEDS :
+            case OpTypes.RANGE_PRECEDES :
+            case OpTypes.RANGE_SUCCEEDS :
             case OpTypes.IN :
             case OpTypes.MATCH_SIMPLE :
             case OpTypes.MATCH_PARTIAL :
