@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -323,6 +323,8 @@ public class ExpressionOp extends Expression {
             case OpTypes.CONCAT_WS :
                 sb.append(Tokens.T_CONCAT_WS).append(' ');
                 break;
+
+            default :
         }
 
         if (getLeftNode() != null) {
@@ -353,6 +355,8 @@ public class ExpressionOp extends Expression {
             case OpTypes.CASEWHEN :
                 acceptsSequences = false;
                 break;
+
+            default :
         }
 
         for (int i = 0; i < nodes.length; i++) {
@@ -485,7 +489,7 @@ public class ExpressionOp extends Expression {
                 // the parent evaluates to true), while its right node is case 2
                 // (how to get the value when the condition in
                 // the parent evaluates to false).
-                resolveTypesForCaseWhen(session);
+                resolveTypesForCaseWhen(session, parent);
                 break;
 
             case OpTypes.CONCAT_WS :
@@ -497,6 +501,7 @@ public class ExpressionOp extends Expression {
                 break;
 
             case OpTypes.ALTERNATIVE :
+                resolveTypesForAlternative(session);
                 break;
 
             case OpTypes.LIMIT :
@@ -529,25 +534,54 @@ public class ExpressionOp extends Expression {
         }
     }
 
+    void resolveTypesForAlternative(Session session) {
+
+        if (nodes[LEFT].dataType == null) {
+            nodes[LEFT].dataType = nodes[RIGHT].dataType;
+        }
+
+        if (nodes[RIGHT].dataType == null) {
+            nodes[RIGHT].dataType = nodes[LEFT].dataType;
+        }
+
+        if (exprSubType == OpTypes.CAST) {
+            if (nodes[RIGHT].dataType == null) {
+                nodes[RIGHT].dataType = nodes[LEFT].dataType =
+                    Type.SQL_VARCHAR_DEFAULT;;
+            }
+
+            dataType = nodes[RIGHT].dataType;
+
+            if (!nodes[RIGHT].dataType.equals(nodes[LEFT].dataType)) {
+                if (dataType.isCharacterType()) {
+                    dataType = Type.SQL_VARCHAR_DEFAULT;
+                }
+
+                nodes[LEFT] = new ExpressionOp(nodes[LEFT], dataType);
+            }
+        } else {
+            dataType = Type.getAggregateType(nodes[LEFT].dataType, dataType);
+            dataType = Type.getAggregateType(nodes[RIGHT].dataType, dataType);
+        }
+    }
+
     /**
      * For CASE WHEN and its special cases section 9.3 of the SQL standard
      * on type aggregation is implemented.
      */
-    void resolveTypesForCaseWhen(Session session) {
+    void resolveTypesForCaseWhen(Session session, Expression parent) {
 
-        if (dataType != null) {
-            return;
-        }
+        nodes[RIGHT].resolveTypes(session, this);
 
         Expression expr = this;
 
         while (expr.opType == OpTypes.CASEWHEN) {
-            dataType =
-                Type.getAggregateType(expr.nodes[RIGHT].nodes[LEFT].dataType,
-                                      dataType);
-            dataType =
-                Type.getAggregateType(expr.nodes[RIGHT].nodes[RIGHT].dataType,
-                                      dataType);
+            if (expr.exprSubType == OpTypes.CAST) {
+                dataType = expr.nodes[RIGHT].dataType;
+            } else {
+                dataType = Type.getAggregateType(expr.nodes[RIGHT].dataType,
+                                                 dataType);
+            }
 
             if (expr.nodes[RIGHT].nodes[RIGHT].opType == OpTypes.CASEWHEN) {
                 expr = expr.nodes[RIGHT].nodes[RIGHT];
@@ -559,15 +593,17 @@ public class ExpressionOp extends Expression {
         expr = this;
 
         while (expr.opType == OpTypes.CASEWHEN) {
-            if (expr.nodes[RIGHT].nodes[LEFT].dataType == null) {
-                expr.nodes[RIGHT].nodes[LEFT].dataType = dataType;
+            if (expr.nodes[RIGHT].dataType == null) {
+                expr.nodes[RIGHT].dataType = dataType;
             }
 
             if (expr.nodes[RIGHT].nodes[RIGHT].dataType == null) {
                 expr.nodes[RIGHT].nodes[RIGHT].dataType = dataType;
             }
 
-            expr.nodes[RIGHT].dataType = dataType;
+            if (expr.nodes[RIGHT].nodes[LEFT].dataType == null) {
+                expr.nodes[RIGHT].nodes[LEFT].dataType = dataType;
+            }
 
             if (expr.nodes[RIGHT].nodes[RIGHT].opType == OpTypes.CASEWHEN) {
                 expr = expr.nodes[RIGHT].nodes[RIGHT];
@@ -596,61 +632,11 @@ public class ExpressionOp extends Expression {
             expr = expr.nodes[RIGHT].nodes[RIGHT];
         }
 
-        if (exprSubType == OpTypes.CAST) {
-            if (nodes[RIGHT].nodes[RIGHT].dataType != null
-                    && nodes[RIGHT].nodes[RIGHT].dataType
-                       != nodes[RIGHT].nodes[LEFT].dataType) {
-                Type castType = nodes[RIGHT].nodes[RIGHT].dataType;
-
-                if (castType.isCharacterType()) {
-                    castType = Type.SQL_VARCHAR_DEFAULT;
-                }
-
-                nodes[RIGHT].nodes[LEFT] =
-                    new ExpressionOp(nodes[RIGHT].nodes[LEFT], castType);
+        if (parent == null || parent.opType != OpTypes.ALTERNATIVE) {
+            if (dataType == null || dataType.typeCode == Types.SQL_ALL_TYPES) {
+                throw Error.error(ErrorCode.X_42567);
             }
         }
-
-        expr = this;
-
-        while (expr.opType == OpTypes.CASEWHEN) {
-            dataType =
-                Type.getAggregateType(expr.nodes[RIGHT].nodes[LEFT].dataType,
-                                      dataType);
-            dataType =
-                Type.getAggregateType(expr.nodes[RIGHT].nodes[RIGHT].dataType,
-                                      dataType);
-
-            if (expr.nodes[RIGHT].nodes[RIGHT].opType == OpTypes.CASEWHEN) {
-                expr = expr.nodes[RIGHT].nodes[RIGHT];
-            } else {
-                expr = expr.nodes[RIGHT].nodes[LEFT];
-            }
-        }
-
-        expr = this;
-
-        while (expr.opType == OpTypes.CASEWHEN) {
-            if (expr.nodes[RIGHT].nodes[LEFT].dataType == null) {
-                expr.nodes[RIGHT].nodes[LEFT].dataType = dataType;
-            }
-
-            if (expr.nodes[RIGHT].nodes[RIGHT].dataType == null) {
-                expr.nodes[RIGHT].nodes[RIGHT].dataType = dataType;
-            }
-
-            expr.nodes[RIGHT].dataType              = dataType;
-
-            if (expr.nodes[RIGHT].nodes[RIGHT].opType == OpTypes.CASEWHEN) {
-                expr = expr.nodes[RIGHT].nodes[RIGHT];
-            } else {
-                expr = expr.nodes[RIGHT].nodes[LEFT];
-            }
-        }
-
-        if (dataType == null || dataType.typeCode == Types.SQL_ALL_TYPES) {
-            throw Error.error(ErrorCode.X_42567);
-        }
     }
 
     public Object getValue(Session session) {
