@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,7 @@ package org.hsqldb;
 
 import java.math.BigDecimal;
 
+import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.ArrayUtil;
@@ -47,7 +48,7 @@ import org.hsqldb.types.Types;
 
 /**
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.9.0
  */
 public class ParserBase {
@@ -56,21 +57,24 @@ public class ParserBase {
     protected Token   token;
 
     //
-    protected boolean       isRecording;
-    protected HsqlArrayList recordedStatement;
-    private final Token     dummyToken = new Token();
+    private final Token dummyToken = new Token();
 
     //
-    protected boolean isCheckOrTriggerCondition;
-    protected boolean isSchemaDefinition;
-    protected int     parsePosition;
+    protected int           parsePosition;
+    protected HsqlException lastError;
+    protected HsqlName      lastSynonym;
+    protected boolean       isCheckOrTriggerCondition;
+    protected boolean       isSchemaDefinition;
+    protected boolean       isViewDefinition;
+    protected boolean       isRecording;
+    protected HsqlArrayList recordedStatement;
     static final BigDecimal LONG_MAX_VALUE_INCREMENT =
         BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.valueOf(1));
 
     /**
      * Constructs a new BaseParser object with the given context.
      *
-     * @param t the token source from which to parse commands
+     * @param scanner the token source from which to parse commands
      */
     ParserBase(Scanner scanner) {
         this.scanner = scanner;
@@ -97,14 +101,17 @@ public class ParserBase {
      *
      * @param sql a new SQL character sequence to replace the current one
      */
-    void reset(String sql) {
+    void reset(Session session, String sql) {
 
-        scanner.reset(sql);
+        scanner.reset(session, sql);
 
         //
         parsePosition             = 0;
+        lastError                 = null;
+        lastSynonym               = null;
         isCheckOrTriggerCondition = false;
         isSchemaDefinition        = false;
+        isViewDefinition          = false;
         isRecording               = false;
         recordedStatement         = null;
     }
@@ -279,6 +286,8 @@ public class ParserBase {
                 case Tokens.X_MALFORMED_IDENTIFIER :
                     errorCode = ErrorCode.X_42583;
                     break;
+
+                default :
             }
 
             throw Error.error(errorCode, token.getFullString());
@@ -358,7 +367,7 @@ public class ParserBase {
         }
     }
 
-    void checkIsUndelimitedIdentifer() {
+    void checkIsUndelimitedIdentifier() {
 
         if (!token.isUndelimitedIdentifier) {
             throw unexpectedToken();
@@ -372,10 +381,10 @@ public class ParserBase {
         }
     }
 
-    void checkIsValue(int dataTypeCode) {
+    void checkIsQuotedString() {
 
         if (token.tokenType != Tokens.X_VALUE
-                || token.dataType.typeCode != dataTypeCode) {
+                || !token.dataType.isCharacterType()) {
             throw unexpectedToken();
         }
     }
@@ -423,7 +432,7 @@ public class ParserBase {
 
         checkIsValue();
 
-        if (token.dataType.typeCode != Types.SQL_CHAR) {
+        if (!token.dataType.isCharacterType()) {
             throw Error.error(ErrorCode.X_42563);
         }
 
@@ -456,6 +465,28 @@ public class ParserBase {
         return false;
     }
 
+    void readThis(String tokenString) {
+
+        if (!tokenString.equals(token.tokenString)) {
+            String required = tokenString;
+
+            throw unexpectedTokenRequire(required);
+        }
+
+        read();
+    }
+
+    boolean readIfThis(String tokenString) {
+
+        if (tokenString.equals(token.tokenString)) {
+            read();
+
+            return true;
+        }
+
+        return false;
+    }
+
     Integer readIntegerObject() {
 
         int value = readInteger();
@@ -467,7 +498,7 @@ public class ParserBase {
 
         boolean minus = false;
 
-        if (token.tokenType == Tokens.MINUS) {
+        if (token.tokenType == Tokens.MINUS_OP) {
             minus = true;
 
             read();
@@ -502,7 +533,7 @@ public class ParserBase {
 
         boolean minus = false;
 
-        if (token.tokenType == Tokens.MINUS) {
+        if (token.tokenType == Tokens.MINUS_OP) {
             minus = true;
 
             read();
@@ -543,7 +574,7 @@ public class ParserBase {
                 read();
 
                 if (token.tokenType != Tokens.X_VALUE
-                        || token.dataType.typeCode != Types.SQL_CHAR) {
+                        || !token.dataType.isCharacterType()) {
                     break;
                 }
 
@@ -559,7 +590,7 @@ public class ParserBase {
                 read();
 
                 if (token.tokenType != Tokens.X_VALUE
-                        || token.dataType.typeCode != Types.SQL_CHAR) {
+                        || !token.dataType.isCharacterType()) {
                     break;
                 }
 
@@ -576,7 +607,7 @@ public class ParserBase {
                 read();
 
                 if (token.tokenType != Tokens.X_VALUE
-                        || token.dataType.typeCode != Types.SQL_CHAR) {
+                        || !token.dataType.isCharacterType()) {
                     break;
                 }
 
@@ -594,11 +625,11 @@ public class ParserBase {
 
                 read();
 
-                if (token.tokenType == Tokens.MINUS) {
+                if (token.tokenType == Tokens.MINUS_OP) {
                     read();
 
                     minus = true;
-                } else if (token.tokenType == Tokens.PLUS) {
+                } else if (token.tokenType == Tokens.PLUS_OP) {
                     read();
                 }
 
@@ -609,8 +640,8 @@ public class ParserBase {
                 String s = token.tokenString;
 
                 /* INT literal accepted in addition to string literal */
-                if (token.dataType.typeCode != Types.SQL_INTEGER
-                        && token.dataType.typeCode != Types.SQL_CHAR) {
+                if (!token.dataType.isIntegralType()
+                        && !token.dataType.isCharacterType()) {
                     break;
                 }
 
@@ -731,9 +762,9 @@ public class ParserBase {
     static {
 
         // comparison
-        expressionTypeMap.put(Tokens.EQUALS, OpTypes.EQUAL);
-        expressionTypeMap.put(Tokens.GREATER, OpTypes.GREATER);
-        expressionTypeMap.put(Tokens.LESS, OpTypes.SMALLER);
+        expressionTypeMap.put(Tokens.EQUALS_OP, OpTypes.EQUAL);
+        expressionTypeMap.put(Tokens.GREATER_OP, OpTypes.GREATER);
+        expressionTypeMap.put(Tokens.LESS_OP, OpTypes.SMALLER);
         expressionTypeMap.put(Tokens.GREATER_EQUALS, OpTypes.GREATER_EQUAL);
         expressionTypeMap.put(Tokens.LESS_EQUALS, OpTypes.SMALLER_EQUAL);
         expressionTypeMap.put(Tokens.NOT_EQUALS, OpTypes.NOT_EQUAL);
