@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ import org.hsqldb.types.Types;
  * Parser for session and management statements
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.9.0
  */
 public class ParserCommand extends ParserDDL {
@@ -79,7 +79,7 @@ public class ParserCommand extends ParserDDL {
         HsqlArrayList list = new HsqlArrayList();
         Statement     cs   = null;
 
-        reset(sql);
+        reset(session, sql);
 
         while (true) {
             if (token.tokenType == Tokens.X_ENDPARSE) {
@@ -296,6 +296,10 @@ public class ParserCommand extends ParserDDL {
                 cs = compileDeclare();
                 break;
 
+            case Tokens.PERFORM :
+                cs = compilePerform();
+                break;
+
             default :
                 throw unexpectedToken();
         }
@@ -323,8 +327,8 @@ public class ParserCommand extends ParserDDL {
 
     private Statement compileDeclare() {
 
-        Statement    cs;
-        ColumnSchema variables[];
+        Statement      cs;
+        ColumnSchema[] variables;
 
         cs = compileDeclareLocalTableOrNull();
 
@@ -530,8 +534,6 @@ public class ParserCommand extends ParserDDL {
 
     private Statement compileSet() {
 
-        int position = super.getPosition();
-
         read();
 
         switch (token.tokenType) {
@@ -771,7 +773,7 @@ public class ParserCommand extends ParserDDL {
 
                     read();
                 } else {
-                    delay = this.readInteger();
+                    delay = readInteger();
 
                     if (delay < 0) {
                         delay = 0;
@@ -784,7 +786,7 @@ public class ParserCommand extends ParserDDL {
                     }
                 }
 
-                Object[] args = new Object[]{ new Integer(delay) };
+                Object[] args = new Object[]{ Integer.valueOf(delay) };
 
                 return new StatementCommand(
                     StatementTypes.SET_DATABASE_FILES_WRITE_DELAY, args, null,
@@ -792,12 +794,12 @@ public class ParserCommand extends ParserDDL {
             }
             case Tokens.PASSWORD : {
                 String  password;
-                boolean isDigest = false;
+                Boolean isDigest = Boolean.FALSE;
 
                 read();
 
                 if (readIfThis(Tokens.DIGEST)) {
-                    isDigest = true;
+                    isDigest = Boolean.TRUE;
                 }
 
                 password = readPassword();
@@ -860,7 +862,7 @@ public class ParserCommand extends ParserDDL {
 
         Table    table = readTableName();
         Object[] args  = new Object[] {
-            table.getName(), null
+            table.getName(), null, null
         };
 
         switch (token.tokenType) {
@@ -916,6 +918,7 @@ public class ParserCommand extends ParserDDL {
                 read();
 
                 args[1] = value;
+                args[2] = Integer.valueOf(TableBase.CACHED_TABLE);
 
                 return new StatementCommand(StatementTypes.SET_TABLE_INDEX,
                                             args, null,
@@ -944,6 +947,7 @@ public class ParserCommand extends ParserDDL {
 
                     case TableBase.MEMORY_TABLE :
                     case TableBase.CACHED_TABLE :
+                    case TableBase.TEXT_TABLE :
                         break;
 
                     default :
@@ -952,7 +956,7 @@ public class ParserCommand extends ParserDDL {
 
                 read();
 
-                args[1] = new Integer(newType);
+                args[1] = Integer.valueOf(newType);
 
                 return new StatementCommand(StatementTypes.SET_TABLE_TYPE,
                                             args, null,
@@ -993,7 +997,7 @@ public class ParserCommand extends ParserDDL {
             case Tokens.SPACE : {
                 read();
 
-                Integer id = super.readIntegerObject();
+                Integer id = readIntegerObject();
 
                 args = new Object[] {
                     table.getName(), id
@@ -1081,7 +1085,7 @@ public class ParserCommand extends ParserDDL {
 
                 Integer  value = readIntegerObject();
                 Object[] args  = new Object[] {
-                    value, sqlLog
+                    value, Boolean.valueOf(sqlLog)
                 };
 
                 return new StatementCommand(
@@ -1149,6 +1153,18 @@ public class ParserCommand extends ParserDDL {
 
                 switch (token.tokenType) {
 
+                    case Tokens.LIVE :
+                        read();
+                        readThis(Tokens.OBJECT);
+
+                        property = HsqlDatabaseProperties.sql_live_object;
+                        flag     = processTrueOrFalseObject();
+
+                        Object[] args = new Object[] {
+                            property, flag, value
+                        };
+                        break;
+
                     case Tokens.NAMES :
                         read();
 
@@ -1209,6 +1225,14 @@ public class ParserCommand extends ParserDDL {
                             HsqlDatabaseProperties.jdbc_translate_tti_types;
                         break;
 
+                    case Tokens.CHARACTER :
+                        read();
+                        readThis(Tokens.LITERAL);
+
+                        flag     = processTrueOrFalseObject();
+                        property = HsqlDatabaseProperties.sql_char_literal;
+                        break;
+
                     case Tokens.CONCAT_WORD :
                         read();
                         readThis(Tokens.NULLS);
@@ -1371,6 +1395,8 @@ public class ParserCommand extends ParserDDL {
 
                         mode = TransactionManager.LOCKS;
                         break;
+
+                    default :
                 }
 
                 HsqlName[] names =
@@ -1385,7 +1411,10 @@ public class ParserCommand extends ParserDDL {
             case Tokens.UNIQUE : {
                 read();
                 readThis(Tokens.NAME);
-                isUndelimitedSimpleName();
+
+                if (!isUndelimitedSimpleName()) {
+                    throw unexpectedToken();
+                }
 
                 name = token.tokenString;
 
@@ -1429,19 +1458,20 @@ public class ParserCommand extends ParserDDL {
             case Tokens.CHECK : {
                 read();
 
-                long longValue = readBigint();
+                long longValue  = readBigint();
+                long longValue2 = -1;
 
                 type  = StatementTypes.SET_DATABASE_FILES_CHECK;
                 names = database.schemaManager.getCatalogNameArray();
 
                 if (readIfThis(Tokens.COMMA)) {
-                    readBigint();
+                    longValue2 = readBigint();
                 }
 
                 Object[] args = new Object[2];
 
                 args[0] = Long.valueOf(longValue);
-                args[1] = Long.valueOf(0);
+                args[1] = Long.valueOf(longValue2);
 
                 return new StatementCommand(type, args, null, names);
             }
@@ -1583,7 +1613,7 @@ public class ParserCommand extends ParserDDL {
 
                     read();
                 } else {
-                    delay = this.readInteger();
+                    delay = readInteger();
 
                     if (delay < 0) {
                         delay = 0;
@@ -1596,7 +1626,7 @@ public class ParserCommand extends ParserDDL {
                     }
                 }
 
-                value = new Integer(delay);
+                value = Integer.valueOf(delay);
 
                 break;
             }
@@ -1607,11 +1637,11 @@ public class ParserCommand extends ParserDDL {
                 if (token.tokenType == Tokens.TEXT) {
                     read();
 
-                    value = new Integer(0);
+                    value = Integer.valueOf(0);
                 } else {
                     readThis(Tokens.COMPRESSED);
 
-                    value = new Integer(3);
+                    value = Integer.valueOf(3);
                 }
 
                 type = StatementTypes.SET_DATABASE_FILES_SCRIPT_FORMAT;
@@ -1624,8 +1654,8 @@ public class ParserCommand extends ParserDDL {
 
         Object[] args = new Object[2];
 
-        args[0] = flag == null ? (Object) value
-                               : (Object) flag;
+        args[0] = flag == null ? value
+                               : flag;
         args[1] = mode;
 
         return new StatementCommand(type, args, null, names);
@@ -1705,7 +1735,7 @@ public class ParserCommand extends ParserDDL {
                             throw unexpectedToken();
                     }
 
-                    args[1] = new Integer(level);
+                    args[1] = Integer.valueOf(level);
 
                     break;
                 }
@@ -1812,7 +1842,7 @@ public class ParserCommand extends ParserDDL {
 
         outerloop:
         while (true) {
-            Table table = readTableName();
+            Table table = readTableName(true);
 
             switch (token.tokenType) {
 
@@ -1875,7 +1905,6 @@ public class ParserCommand extends ParserDDL {
 
             read();
 
-            String   sql  = getLastPart();
             Object[] args = new Object[]{ savepoint };
             Statement cs =
                 new StatementSession(StatementTypes.ROLLBACK_SAVEPOINT, args);
@@ -1917,7 +1946,6 @@ public class ParserCommand extends ParserDDL {
 
         read();
 
-        String   sql  = getLastPart();
         Object[] args = new Object[]{ name };
 
         return new StatementSession(StatementTypes.SAVEPOINT, args);
@@ -1932,7 +1960,6 @@ public class ParserCommand extends ParserDDL {
 
         read();
 
-        String   sql  = getLastPart();
         Object[] args = new Object[]{ name };
 
         return new StatementSession(StatementTypes.RELEASE_SAVEPOINT, args);
@@ -2032,8 +2059,6 @@ public class ParserCommand extends ParserDDL {
             }
         }
 
-        String sql = getLastPart();
-
         return new StatementSession(StatementTypes.SET_ROLE,
                                     new Expression[]{ e });
     }
@@ -2066,8 +2091,6 @@ public class ParserCommand extends ParserDDL {
             }
         }
 
-        String sql = getLastPart();
-
         return new StatementSession(StatementTypes.SET_TIME_ZONE,
                                     new Expression[]{ e });
     }
@@ -2102,9 +2125,10 @@ public class ParserCommand extends ParserDDL {
                 read();
                 break;
 
-            // only semicolon is accepted here
+            default :
         }
 
+        // only semicolon is accepted here
         if (token.tokenType == Tokens.SEMICOLON) {
             read();
         }
@@ -2113,8 +2137,7 @@ public class ParserCommand extends ParserDDL {
             throw unexpectedToken();
         }
 
-        String   sql  = getLastPart();
-        Object[] args = new Object[]{ new Integer(closemode) };
+        Object[] args = new Object[]{ Integer.valueOf(closemode) };
         Statement cs = new StatementCommand(StatementTypes.DATABASE_SHUTDOWN,
                                             args, null, null);
 
@@ -2248,6 +2271,35 @@ public class ParserCommand extends ParserDDL {
         return cs;
     }
 
+    private Statement compilePerform() {
+
+        read();
+
+        switch (token.tokenType) {
+
+            case Tokens.CHECK : {
+                read();
+                readThis(Tokens.TABLE);
+
+                Table table = readTableName();
+
+                readThis(Tokens.INDEX);
+
+                Object[] args = new Object[] {
+                    table.getName(), Integer.valueOf(1), null
+                };
+                HsqlName[] names =
+                    database.schemaManager.getCatalogAndBaseTableNames(
+                        table.getName());
+
+                return new StatementCommand(StatementTypes.CHECK_INDEX, args,
+                                            null, names);
+            }
+            default :
+                throw unexpectedToken();
+        }
+    }
+
     private Statement compileCheckpoint() {
 
         boolean defrag = false;
