@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,7 @@ import org.hsqldb.types.Types;
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  *
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.9.0
  */
 public class QuerySpecification extends QueryExpression {
@@ -167,15 +167,6 @@ public class QuerySpecification extends QueryExpression {
         rangeVariableList.add(rangeVar);
     }
 
-    void addRangeSeparator() {
-
-        RangeVariable range =
-            (RangeVariable) rangeVariableList.get(rangeVariableList.size()
-                - 1);
-
-        range.isBoundary = true;
-    }
-
     public TableDerived getValueListTable() {
 
         if (isValueList) {
@@ -458,7 +449,7 @@ public class QuerySpecification extends QueryExpression {
 
     /**
      * Resolves all column expressions in the GROUP BY clause and beyond.
-     * Replaces any alias column expression in the ORDER BY cluase
+     * Replaces any alias column expression in the ORDER BY clause
      * with the actual select column expression.
      */
     private void resolveColumnReferences(Session session,
@@ -524,10 +515,10 @@ public class QuerySpecification extends QueryExpression {
                                                rangeGroups, false);
         }
 
-        resolveColumnRefernecesInOrderBy(session, rangeGroups, sortAndSlice);
+        resolveColumnReferencesInOrderBy(session, rangeGroups, sortAndSlice);
     }
 
-    void resolveColumnRefernecesInOrderBy(Session session,
+    void resolveColumnReferencesInOrderBy(Session session,
                                           RangeGroup[] rangeGroups,
                                           SortAndSlice sortAndSlice) {
 
@@ -772,7 +763,7 @@ public class QuerySpecification extends QueryExpression {
         Expression     result          = null;
         OrderedHashSet joinColumnNames = new OrderedHashSet();
 
-        for (int i = 0; i < rangeVariableList.size(); i++) {
+        for (int i = rangeVariableList.size() - 1; i >= 0; i--) {
             RangeVariable  range = (RangeVariable) rangeVariableList.get(i);
             HashMappedList columnList = range.rangeTable.columnList;
 
@@ -800,9 +791,6 @@ public class QuerySpecification extends QueryExpression {
                 Expression e = new ExpressionLogical(range, leftPosition,
                                                      rightRange,
                                                      rightPosition);
-
-                result = ExpressionLogical.andExpressions(result, e);
-
                 ExpressionColumn col = range.getColumnExpression(name);
 
                 if (col == null) {
@@ -810,14 +798,31 @@ public class QuerySpecification extends QueryExpression {
                         e.getLeftNode(), e.getRightNode()
                     }, name);
 
-                    range.addNamedJoinColumnExpression(name, col);
-                } else {
+                    range.addNamedJoinColumnExpression(name, col,
+                                                       leftPosition);
+
+                    result = ExpressionLogical.andExpressions(result, e);
+
+                    rightRange.addNamedJoinColumnExpression(name, col,
+                            rightPosition);
+                } else if (rightRange.getColumnExpression(name) == null
+                           && (!range.isLeftJoin || range.isRightJoin)) {
+                    if (range.isLeftJoin && range.isRightJoin) {
+                        e = new ExpressionLogical(col, e.getRightNode());
+                    }
+
                     col.nodes = (Expression[]) ArrayUtil.resizeArray(col.nodes,
                             col.nodes.length + 1);
                     col.nodes[col.nodes.length - 1] = e.getRightNode();
-                }
+                    result = ExpressionLogical.andExpressions(result, e);
 
-                rightRange.addNamedJoinColumnExpression(name, col);
+                    rightRange.addNamedJoinColumnExpression(name, col,
+                            rightPosition);
+                }
+            }
+
+            if (!range.isJoin) {
+                break;
             }
         }
 
@@ -1036,8 +1041,7 @@ public class QuerySpecification extends QueryExpression {
         }
 
         for (int i = 0; i < rangeVariables.length; i++) {
-            if (rangeVariables[i].isLateral || rangeVariables[i].isLeftJoin
-                    || rangeVariables[i].isRightJoin) {
+            if (rangeVariables[i].isRightJoin) {
                 return;
             }
         }
@@ -1088,6 +1092,10 @@ public class QuerySpecification extends QueryExpression {
             }
         }
 
+        if (range != rangeVariables[0]) {
+            return;
+        }
+
         boolean check = ArrayUtil.areAllIntIndexesAsBooleanArray(colMap,
             range.usedColumns);
 
@@ -1118,6 +1126,7 @@ public class QuerySpecification extends QueryExpression {
                    && aggregateSet.size() == 1 && indexLimitVisible == 1) {
             Expression e      = exprColumns[indexStartAggregates];
             int        opType = e.getType();
+            Expression expr   = e.getLeftNode();
 
             switch (opType) {
 
@@ -1140,19 +1149,15 @@ public class QuerySpecification extends QueryExpression {
                     break;
                 }
                 case OpTypes.COUNT : {
-                    if (e.hasCondition()) {
-                        break;
-                    }
-
-                    if (rangeVariables.length == 1 && queryCondition == null) {
-                        Expression expr = e.getLeftNode();
-
+                    if (!e.hasCondition() && rangeVariables.length == 1
+                            && queryCondition == null) {
                         if (expr.getType() == OpTypes.ASTERISK) {
                             isSimpleCount = true;
+
+                            break;
                         } else if (expr.getNullability()
                                    == SchemaObject.Nullability.NO_NULLS) {
-                            if (((ExpressionAggregate) e)
-                                    .isDistinctAggregate) {
+                            if (e.isDistinctAggregate) {
                                 if (expr.opType == OpTypes.COLUMN) {
                                     Table t =
                                         expr.getRangeVariable().getTable();
@@ -1161,16 +1166,18 @@ public class QuerySpecification extends QueryExpression {
                                         if (t.getColumn(t.getPrimaryKey()[0])
                                                 == expr.getColumn()) {
                                             isSimpleCount = true;
+
+                                            break;
                                         }
                                     }
                                 }
                             } else {
                                 isSimpleCount = true;
+
+                                break;
                             }
                         }
                     }
-
-                    break;
                 }
                 default :
             }
@@ -1200,7 +1207,7 @@ public class QuerySpecification extends QueryExpression {
         //     select cola , sum(colb) from t group by abs(cola) // not allowed because incorrect
         // - group by can introduce invisible, derived columns into the query table
         // - check the having expression can be decomposed into
-        //   select list expresions plus group by expressions
+        //   select list expressions plus group by expressions
         // - having cannot introduce additional, derived columns
         // - having cannot reference columns not in the select or group by list
         // - if there is any aggregate in select list but no group by, no
@@ -1210,7 +1217,7 @@ public class QuerySpecification extends QueryExpression {
         // - if grouped by, then order by should be decomposed into the
         //   select list plus group by list
         // - references to column aliases are allowed only in order by (Standard
-        //   compliance) and take precendence over references to non-alias
+        //   compliance) and take precedence over references to non-alias
         //   column names.
         // - references to table / correlation and column list in correlation
         //   names are handled according to the Standard
@@ -1482,10 +1489,9 @@ public class QuerySpecification extends QueryExpression {
 
     private Result buildResult(Session session, int[] limits) {
 
-        RowSetNavigatorData navigator = new RowSetNavigatorData(session,
-            (QuerySpecification) this);
-        Result  result        = Result.newResult(navigator);
-        boolean resultGrouped = isGrouped && !isSimpleDistinct;
+        RowSetNavigatorData navigator = new RowSetNavigatorData(session, this);
+        Result              result        = Result.newResult(navigator);
+        boolean             resultGrouped = isGrouped && !isSimpleDistinct;
 
         result.metaData = resultMetaData;
 
@@ -1631,6 +1637,14 @@ public class QuerySpecification extends QueryExpression {
 
             if (groupData == null) {
                 navigator.add(data);
+
+                if (isSimpleDistinct) {
+                    for (int i = 1; i < rangeVariables.length; i++) {
+                        rangeIterators[i].reset();
+                    }
+
+                    currentIndex = 0;
+                }
             } else if (isAggregated) {
                 navigator.update(groupData, data);
             }
@@ -1705,7 +1719,7 @@ public class QuerySpecification extends QueryExpression {
 
         if (havingCondition != null) {
             while (navigator.hasNext()) {
-                Object[] data = (Object[]) navigator.getNext();
+                Object[] data = navigator.getNext();
 
                 if (!Boolean.TRUE.equals(
                         data[indexLimitVisible + groupByColumnCount])) {
@@ -1914,12 +1928,9 @@ public class QuerySpecification extends QueryExpression {
             columnList.add(nameString, column);
         }
 
-        try {
-            resultTable = new TableDerived(session.database, tableName,
-                                           tableType, resultColumnTypes,
-                                           columnList,
-                                           ValuePool.emptyIntArray);
-        } catch (Exception e) {}
+        resultTable = new TableDerived(session.database, tableName, tableType,
+                                       resultColumnTypes, columnList,
+                                       ValuePool.emptyIntArray);
     }
 
     public String getSQL() {
@@ -2139,8 +2150,6 @@ public class QuerySpecification extends QueryExpression {
         if (rangeVariables.length != 1) {
             isBaseMergeable = false;
             isMergeable     = false;
-
-            return;
         }
     }
 
