@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -76,7 +76,7 @@ import org.hsqldb.types.Type.TypedComparator;
  * Implementation of SQL sessions.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.7.0
  */
 public class Session implements SessionInterface {
@@ -104,6 +104,7 @@ public class Session implements SessionInterface {
     boolean                 isPreTransaction;
     boolean                 isTransaction;
     boolean                 isBatch;
+    volatile boolean        abortAction;
     volatile boolean        abortTransaction;
     volatile boolean        redoAction;
     HsqlArrayList           rowActionList;
@@ -111,8 +112,8 @@ public class Session implements SessionInterface {
     public OrderedHashSet   waitedSessions;
     public OrderedHashSet   waitingSessions;
     OrderedHashSet          tempSet;
+    OrderedHashSet          actionSet;
     public CountUpDownLatch latch = new CountUpDownLatch();
-    Statement               lockStatement;
     TimeoutManager          timeoutManager;
 
     // current settings
@@ -176,6 +177,7 @@ public class Session implements SessionInterface {
         waitedSessions              = new OrderedHashSet();
         waitingSessions             = new OrderedHashSet();
         tempSet                     = new OrderedHashSet();
+        actionSet                   = new OrderedHashSet();
         isolationLevelDefault       = database.defaultIsolationLevel;
         ignoreCase                  = database.sqlIgnoreCase;
         isolationLevel              = isolationLevelDefault;
@@ -188,7 +190,7 @@ public class Session implements SessionInterface {
                                                  : Boolean.FALSE;
         sessionContext.isReadOnly   = isReadOnlyDefault ? Boolean.TRUE
                                                         : Boolean.FALSE;
-        parser = new ParserCommand(this, new Scanner(database));
+        parser                      = new ParserCommand(this, new Scanner());
 
         setResultMemoryRowCount(database.getResultMaxMemoryRows());
         resetSchema();
@@ -213,6 +215,10 @@ public class Session implements SessionInterface {
         return sessionId;
     }
 
+    public int getRandomId() {
+        return randomId;
+    }
+
     /**
      * Closes this Session.
      */
@@ -231,18 +237,20 @@ public class Session implements SessionInterface {
         sessionData.closeAllNavigators();
         sessionData.persistentStoreCollection.release();
         statementManager.reset();
-        database.sessionManager.removeSession(this);
-        database.closeIfLast();
 
         // keep sessionContext and sessionData
         rowActionList.clear();
 
-        database                    = null;
+        isClosed                    = true;
         user                        = null;
         sessionContext.savepoints   = null;
         sessionContext.lastIdentity = null;
         intConnection               = null;
-        isClosed                    = true;
+
+        database.sessionManager.removeSession(this);
+        database.closeIfLast();
+
+        database = null;
     }
 
     /**
@@ -342,7 +350,7 @@ public class Session implements SessionInterface {
      * @return this Session's User object
      */
     public User getUser() {
-        return (User) user;
+        return user;
     }
 
     public Grantee getGrantee() {
@@ -441,8 +449,11 @@ public class Session implements SessionInterface {
 
 //        tempActionHistory.add("add delete action " + actionTimestamp);
         if (abortTransaction) {
+            throw Error.error(ErrorCode.X_40001);
+        }
 
-//            throw Error.error(ErrorCode.X_40001);
+        if (abortAction) {
+            throw Error.error(ErrorCode.X_40502);
         }
 
         database.txManager.addDeleteAction(this, table, store, row, colMap);
@@ -457,9 +468,16 @@ public class Session implements SessionInterface {
 
         // abort only after adding so that the new row gets removed from indexes
         if (abortTransaction) {
-
-//            throw Error.error(ErrorCode.X_40001);
+            throw Error.error(ErrorCode.X_40001);
         }
+
+        if (abortAction) {
+            throw Error.error(ErrorCode.X_40502);
+        }
+    }
+
+    public HsqlArrayList getRowActionList() {
+        return rowActionList;
     }
 
     /**
@@ -497,6 +515,8 @@ public class Session implements SessionInterface {
     public void endAction(Result result) {
 
 //        tempActionHistory.add("endAction " + actionTimestamp);
+        abortAction = false;
+
         sessionData.persistentStoreCollection.clearStatementTables();
 
         if (result.mode == ResultConstants.ERROR) {
@@ -516,22 +536,9 @@ public class Session implements SessionInterface {
 //        tempActionHistory.add("endAction ends " + actionTimestamp);
     }
 
-    public boolean hasLocks(Statement statement) {
-
-        if (lockStatement == statement) {
-            if (isolationLevel == SessionInterface.TX_REPEATABLE_READ
-                    || isolationLevel == SessionInterface.TX_SERIALIZABLE) {
-                return true;
-            }
-
-            if (statement.getTableNamesForRead().length == 0) {
-                return true;
-            }
-        }
-
-        return false;
-    }
-
+    /**
+     * Explicit start of transaction by user
+     */
     public void startTransaction() {
         database.txManager.beginTransaction(this);
     }
@@ -558,7 +565,7 @@ public class Session implements SessionInterface {
     }
 
     /**
-     * Commits any uncommited transaction this Session may have open
+     * Commits any uncommitted transaction this Session may have open
      *
      * @throws  HsqlException
      */
@@ -592,7 +599,7 @@ public class Session implements SessionInterface {
     }
 
     /**
-     * Rolls back any uncommited transaction this Session may have open.
+     * Rolls back any uncommitted transaction this Session may have open.
      *
      * @throws  HsqlException
      */
@@ -621,6 +628,8 @@ public class Session implements SessionInterface {
 
     private void endTransaction(boolean commit, boolean chain) {
 
+        abortTransaction = false;
+
         sessionContext.resetStack();
         sessionContext.savepoints.clear();
         sessionContext.savepointTimestamps.clear();
@@ -629,8 +638,6 @@ public class Session implements SessionInterface {
         sessionData.closeAllTransactionNavigators();
         sessionData.clearLobOps();
 
-        lockStatement = null;
-
         if (!chain) {
             sessionContext.isReadOnly = isReadOnlyDefault ? Boolean.TRUE
                                                           : Boolean.FALSE;
@@ -655,6 +662,7 @@ public class Session implements SessionInterface {
 
     /**
      * Clear structures and reset variables to original. For JDBC use only.
+     * Note: sets autocommit true
      */
     public synchronized void resetSession() {
 
@@ -739,8 +747,6 @@ public class Session implements SessionInterface {
             return;
         }
 
-        String name = (String) sessionContext.savepoints.getKey(0);
-
         database.txManager.rollbackSavepoint(this, 0);
     }
 
@@ -875,7 +881,7 @@ public class Session implements SessionInterface {
     }
 
     /**
-     * Retreives the external JDBC connection
+     * Retrieves the external JDBC connection
      */
     public JDBCConnection getJDBCConnection() {
         return extConnection;
@@ -886,14 +892,14 @@ public class Session implements SessionInterface {
     }
 
     public String getDatabaseUniqueName() {
-        return database.getUniqueName();
+        return database.getNameString();
     }
 
 // boucherb@users 20020810 metadata 1.7.2
 //----------------------------------------------------------------
     private final long connectTime = System.currentTimeMillis();
 
-// more effecient for MetaData concerns than checkAdmin
+// more efficient for MetaData concerns than checkAdmin
 
     /**
      * Getter for admin attribute.
@@ -914,7 +920,7 @@ public class Session implements SessionInterface {
     }
 
     /**
-     * Count of acctions in current transaction.
+     * Count of actions in current transaction.
      *
      * @return the current value
      */
@@ -928,7 +934,7 @@ public class Session implements SessionInterface {
 
     public Statement compileStatement(String sql, int props) {
 
-        parser.reset(sql);
+        parser.reset(this, sql);
 
         Statement cs = parser.compileStatement(props);
 
@@ -937,7 +943,7 @@ public class Session implements SessionInterface {
 
     public Statement compileStatement(String sql) {
 
-        parser.reset(sql);
+        parser.reset(this, sql);
 
         Statement cs =
             parser.compileStatement(ResultProperties.defaultPropsValue);
@@ -1038,11 +1044,6 @@ public class Session implements SessionInterface {
                 } catch (Throwable t) {
                     String errorString = cmd.getMainString();
 
-                    if (database.getProperties().getErrorLevel()
-                            == HsqlDatabaseProperties.NO_MESSAGE) {
-                        errorString = null;
-                    }
-
                     return Result.newErrorResult(t, errorString);
                 }
 
@@ -1285,9 +1286,7 @@ public class Session implements SessionInterface {
         Result r;
 
         if (abortTransaction) {
-            rollbackNoCheck(false);
-
-            return Result.newErrorResult(Error.error(ErrorCode.X_40001));
+            return handleAbortTransaction();
         }
 
         if (sessionContext.depth > 0) {
@@ -1335,6 +1334,7 @@ public class Session implements SessionInterface {
             return r;
         }
 
+        repeatLoop:
         while (true) {
             actionIndex = rowActionList.size();
 
@@ -1347,37 +1347,33 @@ public class Session implements SessionInterface {
             }
 
             if (abortTransaction) {
-                rollbackNoCheck(false);
-
-                sessionContext.currentStatement = null;
-
-                return Result.newErrorResult(Error.error(ErrorCode.X_40001));
+                return handleAbortTransaction();
             }
 
             timeoutManager.startTimeout(timeout);
 
-            try {
-                latch.await();
-            } catch (InterruptedException e) {
-                abortTransaction = true;
-            }
+            while (true) {
+                try {
+                    latch.await();
+                } catch (InterruptedException e) {
+                    Thread.interrupted();
 
-            boolean abortAction = timeoutManager.endTimeout();
+                    continue;
+                }
+
+                break;
+            }
 
             if (abortAction) {
                 r = Result.newErrorResult(Error.error(ErrorCode.X_40502));
 
                 endAction(r);
 
-                break;
+                break repeatLoop;
             }
 
             if (abortTransaction) {
-                rollbackNoCheck(false);
-
-                sessionContext.currentStatement = null;
-
-                return Result.newErrorResult(Error.error(ErrorCode.X_40001));
+                return handleAbortTransaction();
             }
 
             database.txManager.beginActionResume(this);
@@ -1393,33 +1389,36 @@ public class Session implements SessionInterface {
                                                   SimpleLog.LOG_NORMAL);
             }
 
-            lockStatement = sessionContext.currentStatement;
-
             //        tempActionHistory.add("sql execute end " + actionTimestamp + " " + rowActionList.size());
             endAction(r);
 
             if (abortTransaction) {
-                rollbackNoCheck(false);
-
-                sessionContext.currentStatement = null;
-
-                return Result.newErrorResult(Error.error(r.getException(),
-                        ErrorCode.X_40001, null));
+                break repeatLoop;
             }
 
             if (redoAction) {
                 redoAction = false;
 
-                try {
-                    latch.await();
-                } catch (InterruptedException e) {
-                    abortTransaction = true;
+                while (true) {
+                    try {
+                        latch.await();
+                    } catch (InterruptedException e) {
+                        Thread.interrupted();
+
+                        continue;
+                    }
+
+                    break;
                 }
             } else {
-                break;
+                break repeatLoop;
             }
         }
 
+        if (abortTransaction) {
+            return handleAbortTransaction();
+        }
+
         if (sessionContext.depth == 0
                 && (sessionContext.isAutoCommit.booleanValue()
                     || cs.isAutoCommitStatement())) {
@@ -1442,6 +1441,15 @@ public class Session implements SessionInterface {
         return r;
     }
 
+    private Result handleAbortTransaction() {
+
+        rollbackNoCheck(false);
+
+        sessionContext.currentStatement = null;
+
+        return Result.newErrorResult(Error.error(ErrorCode.X_40001));
+    }
+
     private Result executeCompiledBatchStatement(Result cmd) {
 
         long      csid;
@@ -1480,7 +1488,7 @@ public class Session implements SessionInterface {
         Result error = null;
 
         while (nav.hasNext()) {
-            Object[] pvals = (Object[]) nav.getNext();
+            Object[] pvals = nav.getNext();
             Result in = executeCompiledStatement(cs, pvals, cmd.queryTimeout);
 
             // On the client side, iterate over the vals and throw
@@ -1505,7 +1513,7 @@ public class Session implements SessionInterface {
                 // to differentiate between things like
                 // stored procedure calls to methods with
                 // void return type and select statements with
-                // a single row/column containg null
+                // a single row/column containing null
                 updateCounts[count++] = ResultConstants.SUCCESS_NO_INFO;
             } else if (in.mode == ResultConstants.CALL_RESPONSE) {
                 updateCounts[count++] = ResultConstants.SUCCESS_NO_INFO;
@@ -1538,7 +1546,7 @@ public class Session implements SessionInterface {
 
         while (nav.hasNext()) {
             Result   in;
-            Object[] data = (Object[]) nav.getNext();
+            Object[] data = nav.getNext();
             String   sql  = (String) data[0];
 
             try {
@@ -1552,7 +1560,7 @@ public class Session implements SessionInterface {
                 // if (t instanceof OutOfMemoryError) {
                 // System.gc();
                 // }
-                // "in" alread equals "err"
+                // "in" already equals "err"
                 // maybe test for OOME and do a gc() ?
                 // t.printStackTrace();
             }
@@ -1565,7 +1573,7 @@ public class Session implements SessionInterface {
                 // to differentiate between things like
                 // stored procedure calls to methods with
                 // void return type and select statements with
-                // a single row/column containg null
+                // a single row/column containing null
                 updateCounts[count++] = ResultConstants.SUCCESS_NO_INFO;
             } else if (in.mode == ResultConstants.CALL_RESPONSE) {
                 updateCounts[count++] = ResultConstants.SUCCESS_NO_INFO;
@@ -1660,8 +1668,8 @@ public class Session implements SessionInterface {
         if (withZone) {
             if (currentTime == null) {
                 int seconds =
-                    (int) (HsqlDateTime.getNormalisedTime(currentMillis))
-                    / 1000;
+                    (int) (HsqlDateTime.getNormalisedTime(
+                        getCalendarGMT(), currentMillis)) / 1000;
                 int nanos = (int) (currentMillis % 1000) * 1000000;
 
                 currentTime = new TimeData(seconds, nanos, getZoneSeconds());
@@ -1672,7 +1680,8 @@ public class Session implements SessionInterface {
             if (localTime == null) {
                 int seconds =
                     (int) (HsqlDateTime.getNormalisedTime(
-                        currentMillis + getZoneSeconds() * 1000)) / 1000;
+                        getCalendarGMT(),
+                        currentMillis + getZoneSeconds() * 1000L)) / 1000;
                 int nanos = (int) (currentMillis % 1000) * 1000000;
 
                 localTime = new TimeData(seconds, nanos, 0);
@@ -1968,6 +1977,18 @@ public class Session implements SessionInterface {
         return DatabaseURL.S_URL_PREFIX + database.getURI();
     }
 
+    public Result cancel(Result result) {
+
+        if (result.getType() == ResultConstants.SQLCANCEL) {
+            if (result.getSessionRandomID()  == randomId) {
+                database.txManager.resetSession(
+                    null, this, TransactionManager.resetSessionAbort);
+            }
+        }
+
+        return Result.updateZeroResult;
+    }
+
     public boolean isProcessingScript() {
         return isProcessingScript;
     }
@@ -2137,6 +2158,7 @@ public class Session implements SessionInterface {
     SimpleDateFormat simpleDateFormatGMT;
     Random           randomGenerator = new Random();
     long             seed            = -1;
+    public final int randomId        = randomGenerator.nextInt();
 
     //
     public TypedComparator getComparator() {
@@ -2178,6 +2200,9 @@ public class Session implements SessionInterface {
             clientProperties.setProperty(
                 HsqlDatabaseProperties.jdbc_translate_tti_types,
                 database.sqlTranslateTTI);
+            clientProperties.setProperty(
+                HsqlDatabaseProperties.sql_live_object,
+                database.sqlLiveObject);
         }
 
         return clientProperties;
@@ -2323,7 +2348,8 @@ public class Session implements SessionInterface {
                 currentTimeout = 0;
                 aborted        = true;
 
-                latch.setCount(0);
+                database.txManager.resetSession(
+                    null, Session.this, TransactionManager.resetSessionAbort);
 
                 return true;
             }
