@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,7 @@ import org.hsqldb.types.Type;
  * Implementation of Statement for SQL session statements.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.9.0
  */
 public class StatementSession extends Statement {
@@ -102,7 +102,7 @@ public class StatementSession extends Statement {
 
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500,
-                                         "StateemntSession");
+                                         "StatementSession");
         }
     }
 
@@ -400,9 +400,8 @@ public class StatementSession extends Statement {
                     Result result = (Result) value;
 
                     if (result.isData()) {
-                        Object[] data =
-                            (Object[]) result.getNavigator().getNext();
-                        boolean single = !result.getNavigator().next();
+                        Object[] data   = result.getNavigator().getNext();
+                        boolean  single = !result.getNavigator().next();
 
                         if (single && data != null && data[0] != null) {
                             value = data[0];
@@ -547,7 +546,7 @@ public class StatementSession extends Statement {
                     }
 
                     if (session.getGrantee().canChangeAuthorisation()) {
-                        session.setUser((User) userObject);
+                        session.setUser(userObject);
                         session.setRole(null);
                         session.resetSchema();
 
@@ -678,6 +677,7 @@ public class StatementSession extends Statement {
                 Table         table           = (Table) parameters[0];
                 HsqlArrayList tempConstraints = (HsqlArrayList) parameters[1];
                 StatementDMQL statement       = (StatementDMQL) parameters[3];
+                Boolean       ifNotExists     = (Boolean) parameters[4];
 
                 try {
                     if (tempConstraints.size() != 0) {
@@ -687,7 +687,17 @@ public class StatementSession extends Statement {
                     }
 
                     table.compile(session, null);
-                    session.sessionContext.addSessionTable(table);
+
+                    try {
+                        session.sessionContext.addSessionTable(table);
+                    } catch (HsqlException e) {
+                        if (ifNotExists != null
+                                && ifNotExists.booleanValue()) {
+                            return Result.updateZeroResult;
+                        } else {
+                            return Result.newErrorResult(e, sql);
+                        }
+                    }
 
                     if (table.hasLobColumn) {
                         throw Error.error(ErrorCode.X_42534);
