@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -137,6 +137,8 @@ public class TableDerived extends Table {
 
                 uniquePredicate = true;
                 break;
+
+            default:
         }
 
         if (dataExpression != null) {
@@ -152,9 +154,8 @@ public class TableDerived extends Table {
             ParserDQL p = new ParserDQL(session, new Scanner(),
                                         session.parser.compileContext);
 
-            p.reset(sql);
+            p.reset(session, sql);
             p.read();
-
             p.compileContext.setCurrentSubquery(tableName.name);
 
             td = p.XreadSubqueryTableBody(tableName, OpTypes.TABLE_SUBQUERY);
@@ -311,7 +312,7 @@ public class TableDerived extends Table {
             ArrayUtil.fillSequence(cols);
         }
 
-        int pkcols[] = uniqueRows ? cols
+        int[] pkcols = uniqueRows ? cols
                                   : null;
 
         createPrimaryKey(null, pkcols, false);
