@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@ import org.hsqldb.persist.PersistentStore;
  * Manages rows involved in transactions
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 2.0.0
  */
 public class TransactionManagerMVCC extends TransactionManagerCommon
@@ -72,7 +72,7 @@ implements TransactionManager {
 
         database     = db;
         lobSession   = database.sessionManager.getSysLobSession();
-        rowActionMap = new LongKeyHashMap(10000);
+        rowActionMap = new LongKeyHashMap(8192);
         txModel      = MVCC;
     }
 
@@ -80,6 +80,10 @@ implements TransactionManager {
         return globalChangeTimestamp.get();
     }
 
+    public void setGlobalChangeTimestamp(long ts) {
+        globalChangeTimestamp.set(ts);
+    }
+
     public boolean isMVRows() {
         return true;
     }
@@ -88,6 +92,10 @@ implements TransactionManager {
         return true;
     }
 
+    public boolean is2PL() {
+        return false;
+    }
+
     public int getTransactionControl() {
         return MVCC;
     }
@@ -114,7 +122,7 @@ implements TransactionManager {
             for (int i = 0; i < limit; i++) {
                 RowAction action = (RowAction) session.rowActionList.get(i);
 
-                if (!action.canCommit(session, session.tempSet)) {
+                if (!action.canCommit(session, session.actionSet)) {
 
 //                System.out.println("commit conflicts " + session + " " + session.actionTimestamp);
                     return false;
@@ -129,9 +137,9 @@ implements TransactionManager {
                 action.prepareCommit(session);
             }
 
-            for (int i = 0; i < session.tempSet.size(); i++) {
+            for (int i = 0; i < session.actionSet.size(); i++) {
                 Session current =
-                    ((RowActionBase) session.tempSet.get(i)).session;
+                    ((RowActionBase) session.actionSet.get(i)).session;
 
                 current.abortTransaction = true;
             }
@@ -139,7 +147,7 @@ implements TransactionManager {
             return true;
         } finally {
             writeLock.unlock();
-            session.tempSet.clear();
+            session.actionSet.clear();
         }
     }
 
@@ -157,7 +165,7 @@ implements TransactionManager {
             for (int i = 0; i < limit; i++) {
                 RowAction action = (RowAction) session.rowActionList.get(i);
 
-                if (!action.canCommit(session, session.tempSet)) {
+                if (!action.canCommit(session, session.actionSet)) {
 
 //                  System.out.println("commit conflicts " + session + " " + session.actionTimestamp);
                     return false;
@@ -176,9 +184,9 @@ implements TransactionManager {
                 action.commit(session);
             }
 
-            for (int i = 0; i < session.tempSet.size(); i++) {
+            for (int i = 0; i < session.actionSet.size(); i++) {
                 Session current =
-                    ((RowActionBase) session.tempSet.get(i)).session;
+                    ((RowActionBase) session.actionSet.get(i)).session;
 
                 current.abortTransaction = true;
             }
@@ -221,7 +229,7 @@ implements TransactionManager {
             countDownLatches(session);
         } finally {
             writeLock.unlock();
-            session.tempSet.clear();
+            session.actionSet.clear();
         }
 
         return true;
@@ -238,6 +246,7 @@ implements TransactionManager {
 
             rollbackPartial(session, 0, session.transactionTimestamp);
             endTransaction(session);
+            session.logSequences();
             endTransactionTPL(session);
 
             session.isTransaction = false;
@@ -340,7 +349,7 @@ implements TransactionManager {
                 if (session.isolationLevel == SessionInterface
                         .TX_REPEATABLE_READ || session
                         .isolationLevel == SessionInterface.TX_SERIALIZABLE) {
-                    session.tempSet.clear();
+                    session.actionSet.clear();
 
                     session.redoAction       = false;
                     session.abortTransaction = session.txConflictRollback;
@@ -350,7 +359,7 @@ implements TransactionManager {
 
                 // can redo when conflicting action is already committed
                 if (row.rowAction != null && row.rowAction.isDeleted()) {
-                    session.tempSet.clear();
+                    session.actionSet.clear();
 
                     session.redoAction = true;
 
@@ -359,13 +368,13 @@ implements TransactionManager {
                     throw Error.error(ErrorCode.X_40501);
                 }
 
-                redoAction = !session.tempSet.isEmpty();
+                redoAction = !session.actionSet.isEmpty();
 
                 if (redoAction) {
                     actionSession =
-                        ((RowActionBase) session.tempSet.get(0)).session;
+                        ((RowActionBase) session.actionSet.get(0)).session;
 
-                    session.tempSet.clear();
+                    session.actionSet.clear();
 
                     if (actionSession != null) {
                         redoAction = checkDeadlock(session, actionSession);
@@ -416,7 +425,7 @@ implements TransactionManager {
         try {
             store.indexRow(session, row);
         } catch (HsqlException e) {
-            if (session.tempSet.isEmpty()) {
+            if (session.actionSet.isEmpty()) {
                 throw e;
             }
 
@@ -441,11 +450,12 @@ implements TransactionManager {
         try {
             rollbackAction(session);
 
-            RowActionBase otherAction = (RowActionBase) session.tempSet.get(0);
+            RowActionBase otherAction =
+                (RowActionBase) session.actionSet.get(0);
 
             actionSession = otherAction.session;
 
-            session.tempSet.clear();
+            session.actionSet.clear();
 
             if (otherAction.commitTimestamp != 0) {
                 redoWait = false;
@@ -536,15 +546,15 @@ implements TransactionManager {
                 if (result) {
                     session.rowActionList.add(row.rowAction);
                 } else {
-                    if (!session.tempSet.isEmpty()) {
-                        Session current = (Session) session.tempSet.get(0);
+                    if (!session.actionSet.isEmpty()) {
+                        Session current = ((RowActionBase) session.actionSet.get(0)).session;
 
                         session.redoAction = true;
 
                         session.latch.countUp();
                         current.waitingSessions.add(session);
                         session.waitedSessions.add(current);
-                        session.tempSet.clear();
+                        session.actionSet.clear();
 
                         throw Error.error(ErrorCode.X_40501);
                     }
@@ -624,16 +634,7 @@ implements TransactionManager {
      * remove the transaction info
      */
     public void removeTransactionInfo(CachedObject object) {
-
-        if (object.isMemory()) {
-            return;
-        }
-
-        Row row = (Row) object;
-
-        if (row.getTable().tableType == TableBase.CACHED_TABLE) {
-            rowActionMap.remove(row.getPos());
-        }
+        rowActionMap.remove(object.getPos());
     }
 
     public void removeTransactionInfo(long id) {
@@ -715,6 +716,7 @@ implements TransactionManager {
             if (!session.isTransaction) {
                 session.actionTimestamp      = getNextGlobalChangeTimestamp();
                 session.transactionTimestamp = session.actionTimestamp;
+                session.isPreTransaction     = false;
                 session.isTransaction        = true;
 
                 liveTransactionTimestamps.addLast(
@@ -753,12 +755,12 @@ implements TransactionManager {
                 }
             }
 
-            session.isPreTransaction = true;
-
             if (session.abortTransaction) {
                 return;
             }
 
+            session.isPreTransaction = true;
+
             if (!isLockedMode && !cs.isCatalogLock()) {
                 return;
             }
@@ -785,13 +787,12 @@ implements TransactionManager {
             }
 
             session.transactionTimestamp = session.actionTimestamp;
+            session.isPreTransaction     = false;
             session.isTransaction        = true;
 
             liveTransactionTimestamps.addLast(session.transactionTimestamp);
 
             transactionCount++;
-
-            session.isPreTransaction = false;
         } finally {
             writeLock.unlock();
         }
@@ -886,21 +887,6 @@ implements TransactionManager {
         session.waitingSessions.clear();
     }
 
-    void getTransactionSessions(HashSet set) {
-
-        Session[] sessions = database.sessionManager.getAllSessions();
-
-        for (int i = 0; i < sessions.length; i++) {
-            long timestamp = sessions[i].getTransactionTimestamp();
-
-            if (sessions[i].isPreTransaction) {
-                set.add(sessions[i]);
-            } else if (sessions[i].isTransaction) {
-                set.add(sessions[i]);
-            }
-        }
-    }
-
     void endTransactionTPL(Session session) {
 
         if (catalogWriteSession != session) {
@@ -957,8 +943,7 @@ implements TransactionManager {
                 lockTxTs            = session.actionTimestamp;
                 lockSessionId       = session.getId();
 
-                getTransactionSessions(session.tempSet);
-                session.tempSet.remove(session);
+                getTransactionAndPreSessions(session);
 
                 if (!session.tempSet.isEmpty()) {
                     session.waitedSessions.addAll(session.tempSet);
