@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -87,79 +87,80 @@ public class DatabaseInformation {
     protected static final int SYSTEM_SESSIONS              = 22;
     protected static final int SYSTEM_TABLESTATS            = 23;
     protected static final int SYSTEM_TEXTTABLES            = 24;
+    protected static final int SYSTEM_SYNONYMS              = 25;
 
     // SQL 200n tables
-    protected static final int ADMINISTRABLE_ROLE_AUTHORIZATIONS = 25;
-    protected static final int APPLICABLE_ROLES                  = 26;
-    protected static final int ASSERTIONS                        = 27;
-    protected static final int AUTHORIZATIONS                    = 28;
-    protected static final int CHARACTER_SETS                    = 29;
-    protected static final int CHECK_CONSTRAINT_ROUTINE_USAGE    = 30;
-    protected static final int CHECK_CONSTRAINTS                 = 31;
-    protected static final int COLLATIONS                        = 32;
-    protected static final int COLUMN_COLUMN_USAGE               = 33;
-    protected static final int COLUMN_DOMAIN_USAGE               = 34;
-    protected static final int COLUMN_PRIVILEGES                 = 35;
-    protected static final int COLUMN_UDT_USAGE                  = 36;
-    protected static final int COLUMNS                           = 37;
-    protected static final int CONSTRAINT_COLUMN_USAGE           = 38;
-    protected static final int CONSTRAINT_TABLE_USAGE            = 39;
-    protected static final int DATA_TYPE_PRIVILEGES              = 40;
-    protected static final int DOMAIN_CONSTRAINTS                = 41;
-    protected static final int DOMAINS                           = 42;
-    protected static final int ELEMENT_TYPES                     = 43;
-    protected static final int ENABLED_ROLES                     = 44;
-    protected static final int INFORMATION_SCHEMA_CATALOG_NAME   = 45;
-    protected static final int JAR_JAR_USAGE                     = 46;
-    protected static final int JARS                              = 47;
-    protected static final int KEY_COLUMN_USAGE                  = 48;
-    protected static final int METHOD_SPECIFICATIONS             = 49;
-    protected static final int MODULE_COLUMN_USAGE               = 50;
-    protected static final int MODULE_PRIVILEGES                 = 51;
-    protected static final int MODULE_TABLE_USAGE                = 52;
-    protected static final int MODULES                           = 53;
-    protected static final int PARAMETERS                        = 54;
-    protected static final int REFERENTIAL_CONSTRAINTS           = 55;
-    protected static final int ROLE_AUTHORIZATION_DESCRIPTORS    = 56;
-    protected static final int ROLE_COLUMN_GRANTS                = 57;
-    protected static final int ROLE_MODULE_GRANTS                = 58;
-    protected static final int ROLE_ROUTINE_GRANTS               = 59;
-    protected static final int ROLE_TABLE_GRANTS                 = 60;
-    protected static final int ROLE_UDT_GRANTS                   = 61;
-    protected static final int ROLE_USAGE_GRANTS                 = 62;
-    protected static final int ROUTINE_COLUMN_USAGE              = 63;
-    protected static final int ROUTINE_JAR_USAGE                 = 64;
-    protected static final int ROUTINE_PRIVILEGES                = 65;
-    protected static final int ROUTINE_ROUTINE_USAGE             = 66;
-    protected static final int ROUTINE_SEQUENCE_USAGE            = 67;
-    protected static final int ROUTINE_TABLE_USAGE               = 68;
-    protected static final int ROUTINES                          = 69;
-    protected static final int SCHEMATA                          = 70;
-    protected static final int SEQUENCES                         = 71;
-    protected static final int SQL_FEATURES                      = 72;
-    protected static final int SQL_IMPLEMENTATION_INFO           = 73;
-    protected static final int SQL_PACKAGES                      = 74;
-    protected static final int SQL_PARTS                         = 75;
-    protected static final int SQL_SIZING                        = 76;
-    protected static final int SQL_SIZING_PROFILES               = 77;
-    protected static final int TABLE_CONSTRAINTS                 = 78;
-    protected static final int TABLE_PRIVILEGES                  = 79;
-    protected static final int TABLES                            = 80;
-    protected static final int TRANSLATIONS                      = 81;
-    protected static final int TRIGGER_COLUMN_USAGE              = 82;
-    protected static final int TRIGGER_ROUTINE_USAGE             = 83;
-    protected static final int TRIGGER_SEQUENCE_USAGE            = 84;
-    protected static final int TRIGGER_TABLE_USAGE               = 85;
-    protected static final int TRIGGERED_UPDATE_COLUMNS          = 86;
-    protected static final int TRIGGERS                          = 87;
-    protected static final int TYPE_JAR_USAGE                    = 88;
-    protected static final int UDT_PRIVILEGES                    = 89;
-    protected static final int USAGE_PRIVILEGES                  = 90;
-    protected static final int USER_DEFINED_TYPES                = 91;
-    protected static final int VIEW_COLUMN_USAGE                 = 92;
-    protected static final int VIEW_ROUTINE_USAGE                = 93;
-    protected static final int VIEW_TABLE_USAGE                  = 94;
-    protected static final int VIEWS                             = 95;
+    protected static final int ADMINISTRABLE_ROLE_AUTHORIZATIONS = 26;
+    protected static final int APPLICABLE_ROLES                  = 27;
+    protected static final int ASSERTIONS                        = 28;
+    protected static final int AUTHORIZATIONS                    = 29;
+    protected static final int CHARACTER_SETS                    = 30;
+    protected static final int CHECK_CONSTRAINT_ROUTINE_USAGE    = 31;
+    protected static final int CHECK_CONSTRAINTS                 = 32;
+    protected static final int COLLATIONS                        = 33;
+    protected static final int COLUMN_COLUMN_USAGE               = 34;
+    protected static final int COLUMN_DOMAIN_USAGE               = 35;
+    protected static final int COLUMN_PRIVILEGES                 = 36;
+    protected static final int COLUMN_UDT_USAGE                  = 37;
+    protected static final int COLUMNS                           = 38;
+    protected static final int CONSTRAINT_COLUMN_USAGE           = 39;
+    protected static final int CONSTRAINT_TABLE_USAGE            = 40;
+    protected static final int DATA_TYPE_PRIVILEGES              = 41;
+    protected static final int DOMAIN_CONSTRAINTS                = 42;
+    protected static final int DOMAINS                           = 43;
+    protected static final int ELEMENT_TYPES                     = 44;
+    protected static final int ENABLED_ROLES                     = 45;
+    protected static final int INFORMATION_SCHEMA_CATALOG_NAME   = 46;
+    protected static final int JAR_JAR_USAGE                     = 47;
+    protected static final int JARS                              = 48;
+    protected static final int KEY_COLUMN_USAGE                  = 49;
+    protected static final int METHOD_SPECIFICATIONS             = 50;
+    protected static final int MODULE_COLUMN_USAGE               = 51;
+    protected static final int MODULE_PRIVILEGES                 = 52;
+    protected static final int MODULE_TABLE_USAGE                = 53;
+    protected static final int MODULES                           = 54;
+    protected static final int PARAMETERS                        = 55;
+    protected static final int REFERENTIAL_CONSTRAINTS           = 56;
+    protected static final int ROLE_AUTHORIZATION_DESCRIPTORS    = 57;
+    protected static final int ROLE_COLUMN_GRANTS                = 58;
+    protected static final int ROLE_MODULE_GRANTS                = 59;
+    protected static final int ROLE_ROUTINE_GRANTS               = 60;
+    protected static final int ROLE_TABLE_GRANTS                 = 61;
+    protected static final int ROLE_UDT_GRANTS                   = 62;
+    protected static final int ROLE_USAGE_GRANTS                 = 63;
+    protected static final int ROUTINE_COLUMN_USAGE              = 64;
+    protected static final int ROUTINE_JAR_USAGE                 = 65;
+    protected static final int ROUTINE_PRIVILEGES                = 66;
+    protected static final int ROUTINE_ROUTINE_USAGE             = 67;
+    protected static final int ROUTINE_SEQUENCE_USAGE            = 68;
+    protected static final int ROUTINE_TABLE_USAGE               = 69;
+    protected static final int ROUTINES                          = 70;
+    protected static final int SCHEMATA                          = 71;
+    protected static final int SEQUENCES                         = 72;
+    protected static final int SQL_FEATURES                      = 73;
+    protected static final int SQL_IMPLEMENTATION_INFO           = 74;
+    protected static final int SQL_PACKAGES                      = 75;
+    protected static final int SQL_PARTS                         = 76;
+    protected static final int SQL_SIZING                        = 77;
+    protected static final int SQL_SIZING_PROFILES               = 78;
+    protected static final int TABLE_CONSTRAINTS                 = 79;
+    protected static final int TABLE_PRIVILEGES                  = 80;
+    protected static final int TABLES                            = 81;
+    protected static final int TRANSLATIONS                      = 82;
+    protected static final int TRIGGER_COLUMN_USAGE              = 83;
+    protected static final int TRIGGER_ROUTINE_USAGE             = 84;
+    protected static final int TRIGGER_SEQUENCE_USAGE            = 85;
+    protected static final int TRIGGER_TABLE_USAGE               = 86;
+    protected static final int TRIGGERED_UPDATE_COLUMNS          = 87;
+    protected static final int TRIGGERS                          = 88;
+    protected static final int TYPE_JAR_USAGE                    = 89;
+    protected static final int UDT_PRIVILEGES                    = 90;
+    protected static final int USAGE_PRIVILEGES                  = 91;
+    protected static final int USER_DEFINED_TYPES                = 92;
+    protected static final int VIEW_COLUMN_USAGE                 = 93;
+    protected static final int VIEW_ROUTINE_USAGE                = 94;
+    protected static final int VIEW_TABLE_USAGE                  = 95;
+    protected static final int VIEWS                             = 96;
 
     /** system table names strictly in order of their ids */
     protected static final String[] sysTableNames = {
@@ -190,6 +191,7 @@ public class DatabaseInformation {
         "SYSTEM_SESSIONS",                                       //
         "SYSTEM_TABLESTATS",                                     //
         "SYSTEM_TEXTTABLES",                                     //
+        "SYSTEM_SYNONYMS",                                       //
 
         // SQL 200n
         "ADMINISTRABLE_ROLE_AUTHORIZATIONS",                     //
@@ -297,12 +299,12 @@ public class DatabaseInformation {
      * Factory method returns the fullest system table producer
      * implementation available.  This instantiates implementations beginning
      * with the most complete, finally choosing an empty table producer
-     * implemenation (this class) if no better instance can be constructed.
+     * implementation (this class) if no better instance can be constructed.
      * @param db The Database object for which to produce system tables
      * @return the fullest system table producer
      *      implementation available
      */
-    public static final DatabaseInformation newDatabaseInformation(
+    public static DatabaseInformation newDatabaseInformation(
             Database db) {
 
         Class c = null;
