@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@ import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultConstants;
 import org.hsqldb.result.ResultProperties;
 
-/* $Id: JDBCStatement.java 5487 2015-06-06 16:15:53Z fredt $ */
+/* $Id: JDBCStatement.java 5623 2016-05-05 21:03:57Z fredt $ */
 
 // fredt@users   20020320 - patch 1.7.0 - JDBC 2 support and error trapping
 //
@@ -77,7 +77,7 @@ import org.hsqldb.result.ResultProperties;
  * <code>ResultSet</code> object is interleaved
  * with the reading of another, each must have been generated by
  * different <code>Statement</code> objects. All execution methods in the
- * <code>Statement</code> interface implicitly close a statment's current
+ * <code>Statement</code> interface implicitly close a statement's current
  * <code>ResultSet</code> object if an open one exists.
  * <!-- end generic documentation-->
  *
@@ -85,7 +85,7 @@ import org.hsqldb.result.ResultProperties;
  * <div class="ReleaseSpecificDocumentation">
  * <h3>HSQLDB-Specific Information:</h3><p>
  * From version 2.0, the implementation meets the JDBC specification
- * requirment that any existing ResultSet is closed when execute() or
+ * requirement that any existing ResultSet is closed when execute() or
  * executeQuery() methods are called. The connection property close_result=true
  * is required for this behaviour.
  * <p>
@@ -134,7 +134,7 @@ import org.hsqldb.result.ResultProperties;
  *
  * @author Campbell Burnet (boucherb@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.0
+ * @version 2.3.4
  * @revised JDK 1.6, HSQLDB 2.0
  * @see JDBCConnection#createStatement
  * @see JDBCResultSet
@@ -485,8 +485,8 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <div class="ReleaseSpecificDocumentation">
      * <h3>HSQLDB-Specific Information:</h3> <p>
      *
-     * Including 2.0, HSQLDB does <i>not</i> support aborting an SQL
-     * statement; calls to this method are ignored.
+     * HSQLDB version 2.3.4 and later supports aborting an SQL query
+     * or data update statement.
      * </div>
      * <!-- end release-specific documentation -->
      *
@@ -495,8 +495,16 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
      */
-    public synchronized void cancel() throws SQLException {
+    public void cancel() throws SQLException {
         checkClosed();
+        String sql = resultOut.getMainString();
+        Result request = Result.newCancelRequest(-1, sql);
+
+        try {
+            Result response = connection.sessionProxy.cancel(request);
+        } catch (HsqlException e) {
+            throw JDBCUtil.sqlException(e);
+        }
     }
 
     /**
@@ -795,7 +803,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * (JDBC4 clarification:)
      * Gives the JDBC driver a hint as to the number of rows that should
      * be fetched from the database when more rows are needed for
-     * <code>ResultSet</code> objects genrated by this <code>Statement</code>.
+     * <code>ResultSet</code> objects generated by this <code>Statement</code>.
      * If the value specified is zero, then the hint is ignored.
      * The default value is zero.
      * <!-- start generic documentation -->
@@ -922,7 +930,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
 
     /**
      * <!-- start generic documentation -->
-     * Adds the given SQL command to the current list of commmands for this
+     * Adds the given SQL command to the current list of commands for this
      * <code>Statement</code> object. The commands in this list can be
      * executed as a batch by calling the method <code>executeBatch</code>.
      * <P>
@@ -1035,8 +1043,8 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <p>
      * The possible implementations and return values have been modified in
      * the Java 2 SDK, Standard Edition, version 1.3 to
-     * accommodate the option of continuing to proccess commands in a batch
-     * update after a <code>BatchUpdateException</code> obejct has been thrown.
+     * accommodate the option of continuing to process commands in a batch
+     * update after a <code>BatchUpdateException</code> object has been thrown.
      * <!-- end generic documentation -->
      *
      * <!-- start release-specific documentation -->
@@ -1098,7 +1106,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         int[]           updateCounts = new int[navigator.getSize()];
 
         for (int i = 0; i < updateCounts.length; i++) {
-            Object[] data = (Object[]) navigator.getNext();
+            Object[] data = navigator.getNext();
 
             updateCounts[i] = ((Integer) data[0]).intValue();
         }
@@ -1107,8 +1115,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
             if (errorResult == null) {
                 throw new BatchUpdateException(updateCounts);
             } else {
-                errorResult.getMainString();
-
                 throw new BatchUpdateException(errorResult.getMainString(),
                         errorResult.getSubString(),
                         errorResult.getErrorCode(), updateCounts);
@@ -1180,14 +1186,11 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @since JDK 1.4, HSQLDB 1.7
      * @see #execute
      */
-//#ifdef JAVA4
     public synchronized boolean getMoreResults(
             int current) throws SQLException {
         return super.getMoreResults(current);
     }
 
-//#endif JAVA4
-
     /**
      * <!-- start generic documentation -->
      * Retrieves any auto-generated keys created as a result of executing this
@@ -1210,7 +1213,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * the executeUpdate methods that was used is one of the three methods that
      * have the extra parameter indicating return of generated keys<p>
      *
-     * If the executeUpaged method did not specify the columns which represent
+     * If the executeUdate method did not specify the columns which represent
      * the auto-generated keys the IDENTITY column or GENERATED column(s) of the
      * table are returned.<p>
      *
@@ -1221,7 +1224,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      *
      * If column names or indexes provided by the user in the executeUpdate()
      * method calls do not correspond to table columns (incorrect names or
-     * indexes larger than the coloum count), an empty result is returned.
+     * indexes larger than the column count), an empty result is returned.
      *
      * </div>
      * <!-- end release-specific documentation -->
@@ -1233,13 +1236,10 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @throws SQLFeatureNotSupportedException  if the JDBC driver does not support this method
      * @since JDK 1.4, HSQLDB 1.7
      */
-//#ifdef JAVA4
     public synchronized ResultSet getGeneratedKeys() throws SQLException {
         return getGeneratedResultSet();
     }
 
-//#endif JAVA4
-
     /**
      * <!-- start generic documentation -->
      * Executes the given SQL statement and signals the driver with the
@@ -1284,7 +1284,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * this method with a constant of Statement.RETURN_GENERATED_KEYS
      * @since JDK 1.4, HSQLDB 1.7
      */
-//#ifdef JAVA4
     public synchronized int executeUpdate(String sql,
             int autoGeneratedKeys) throws SQLException {
 
@@ -1302,8 +1301,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         return resultIn.getUpdateCount();
     }
 
-//#endif JAVA4
-
     /**
      * <!-- start generic documentation -->
      * Executes the given SQL statement and signals the driver that the
@@ -1344,7 +1341,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @throws SQLFeatureNotSupportedException  if the JDBC driver does not support this method
      * @since JDK 1.4, HSQLDB 1.7
      */
-//#ifdef JAVA4
     public synchronized int executeUpdate(String sql,
             int[] columnIndexes) throws SQLException {
 
@@ -1358,8 +1354,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         return resultIn.getUpdateCount();
     }
 
-//#endif JAVA4
-
     /**
      * <!-- start generic documentation -->
      * Executes the given SQL statement and signals the driver that the
@@ -1399,7 +1393,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @throws SQLFeatureNotSupportedException  if the JDBC driver does not support this method
      * @since JDK 1.4, HSQLDB 1.7
      */
-//#ifdef JAVA4
     public synchronized int executeUpdate(String sql,
             String[] columnNames) throws SQLException {
 
@@ -1413,8 +1406,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         return resultIn.getUpdateCount();
     }
 
-//#endif JAVA4
-
     /**
      * <!-- start generic documentation -->
      * Executes the given SQL statement, which may return multiple results,
@@ -1469,7 +1460,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @see #getGeneratedKeys
      * @since JDK 1.4, HSQLDB 1.7
      */
-//#ifdef JAVA4
     public synchronized boolean execute(
             String sql, int autoGeneratedKeys) throws SQLException {
 
@@ -1483,8 +1473,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         return resultIn.isData();
     }
 
-//#endif JAVA4
-
     /**
      * <!-- start generic documentation -->
      * Executes the given SQL statement, which may return multiple results,
@@ -1536,7 +1524,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @see #getMoreResults
      * @since JDK 1.4, HSQLDB 1.7
      */
-//#ifdef JAVA4
     public synchronized boolean execute(
             String sql, int[] columnIndexes) throws SQLException {
 
@@ -1550,8 +1537,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         return resultIn.isData();
     }
 
-//#endif JAVA4
-
     /**
      * <!-- start generic documentation -->
      * Executes the given SQL statement, which may return multiple results,
@@ -1603,7 +1588,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @see #getGeneratedKeys
      * @since JDK 1.4, HSQLDB 1.7
      */
-//#ifdef JAVA4
     public synchronized boolean execute(
             String sql, String[] columnNames) throws SQLException {
 
@@ -1617,8 +1601,6 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         return resultIn.isData();
     }
 
-//#endif JAVA4
-
     /**
      * <!-- start generic documentation -->
      * Retrieves the result set holdability for <code>ResultSet</code> objects
@@ -1631,12 +1613,10 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * this method is called on a closed <code>Statement</code>
      * @since JDK 1.4, HSQLDB 1.7
      */
-//#ifdef JAVA4
     public synchronized int getResultSetHoldability() throws SQLException {
         return ResultProperties.getJDBCHoldability(rsProperties);
     }
 
-//#endif JAVA4
     //----------------------------- JDBC 4.0 -----------------------------------
 
     /**
@@ -1656,7 +1636,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
     /**
      * Requests that a <code>Statement</code> be pooled or not pooled.  The value
      * specified is a hint to the statement pool implementation indicating
-     * whether the applicaiton wants the statement to be pooled.  It is up to
+     * whether the application wants the statement to be pooled.  It is up to
      * the statement pool manager as to whether the hint is used.
      * <p>
      * The poolable value of a statement is applicable to both internal
