@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -491,7 +491,7 @@ public class FrameworkLogger {
         }
 
         if (log4jLogger == null) {
-            StackTraceElement elements[] = new Throwable().getStackTrace();
+            StackTraceElement[] elements = new Throwable().getStackTrace();
             String            c          = "";
             String            m          = "";
 
