@@ -1,7 +1,7 @@
 /*
  * For work developed by the HSQL Development Group:
  *
- * Copyright (c) 2001-2014, The HSQL Development Group
+ * Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -84,7 +84,7 @@ import org.hsqldb.map.BitMap;
  *
  * @author Thomas Mueller (Hypersonic SQL Group)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.0.1
+ * @version 2.3.4
  * @since 1.7.2
  */
 public class StringConverter {
@@ -345,7 +345,7 @@ public class StringConverter {
      * input is converted into a 7bit escaped ASCII string (output)with the
      * following transformations. All characters outside the 0x20-7f range are
      * converted to a escape sequence and added to output. If a backslash
-     * character is immdediately followed by 'u', the backslash character is
+     * character is immediately followed by 'u', the backslash character is
      * converted to escape sequence and added to output. All the remaining
      * characters in input are added to output without conversion. The escape
      * sequence is backslash, letter u, xxxx, where xxxx is the hex
@@ -364,7 +364,7 @@ public class StringConverter {
             return;
         }
 
-        final int len = s.length();
+        final int len    = s.length();
         int       extras = 0;
 
         if (len == 0) {
@@ -419,7 +419,7 @@ public class StringConverter {
 // fredt@users 20020522 - fix for 557510 - backslash bug
 // this legacy bug resulted from forward reading the input when a backslash
 // was present and manifested itself when a backslash was followed
-// immdediately by a character outside the 0x20-7f range in a database field.
+// immediately by a character outside the 0x20-7f range in a database field.
 
     /**
      * Hsqldb specific decoding used only for log files. This method converts
@@ -758,7 +758,7 @@ public class StringConverter {
                 b.writeNoCheck('#');
                 b.writeBytes(String.valueOf(codePoint));
                 b.writeNoCheck(';');
-            } else if (c < 0x0020 ) {
+            } else if (c < 0x0020) {
                 b.writeNoCheck(' ');
             } else {
                 b.writeNoCheck(c);
@@ -776,9 +776,6 @@ public class StringConverter {
      */
     public static String toStringUUID(byte[] b) {
 
-        char[] chars = new char[36];
-        int    hexIndex;
-
         if (b == null) {
             return null;
         }
@@ -787,6 +784,9 @@ public class StringConverter {
             throw new NumberFormatException();
         }
 
+        char[] chars = new char[36];
+        int    hexIndex;
+
         for (int i = 0, j = 0; i < b.length; ) {
             hexIndex   = (b[i] & 0xf0) >> 4;
             chars[j++] = (char) HEXBYTES[hexIndex];
@@ -811,8 +811,6 @@ public class StringConverter {
      */
     public static byte[] toBinaryUUID(String s) {
 
-        byte[] bytes = new byte[16];
-
         if (s == null) {
             return null;
         }
@@ -821,6 +819,8 @@ public class StringConverter {
             throw new NumberFormatException();
         }
 
+        byte[] bytes = new byte[16];
+
         for (int i = 0, j = 0; i < bytes.length; ) {
             char c    = s.charAt(j++);
             int  high = getNibble(c);
@@ -833,7 +833,9 @@ public class StringConverter {
             if (i >= 4 && i <= 10 && (i % 2) == 0) {
                 c = s.charAt(j++);
 
-                if (c != '-') {}
+                if (c != '-') {
+                    throw new NumberFormatException();
+                }
             }
         }
 
