@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,8 @@
 package org.hsqldb.persist;
 
 import java.util.concurrent.atomic.AtomicLong;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReadWriteLock;
 
 import org.hsqldb.ColumnSchema;
 import org.hsqldb.Database;
@@ -63,18 +65,22 @@ import org.hsqldb.types.Type;
  */
 public abstract class RowStoreAVL implements PersistentStore {
 
-    Database                  database;
-    PersistentStoreCollection manager;
-    TableSpaceManager         tableSpace;
-    Index[]                   indexList    = Index.emptyArray;
-    CachedObject[]            accessorList = CachedObject.emptyArray;
-    TableBase                 table;
-    long                      baseElementCount;
-    AtomicLong                elementCount = new AtomicLong();
-    long                      storageSize;
-    boolean[]                 nullsList;
-    double[][]                searchCost;
-    boolean                   isSchemaStore;
+    Database          database;
+    TableSpaceManager tableSpace;
+    Index[]           indexList    = Index.emptyArray;
+    CachedObject[]    accessorList = CachedObject.emptyArray;
+    TableBase         table;
+    long              baseElementCount;
+    AtomicLong        elementCount = new AtomicLong();
+    long              storageSize;
+    boolean[]         nullsList;
+    double[][]        searchCost;
+    boolean           isSchemaStore;
+
+    //
+    ReadWriteLock lock;
+    Lock          readLock;
+    Lock          writeLock;
 
     // for result tables
     // for INFORMATION SCHEMA tables
@@ -107,8 +113,6 @@ public abstract class RowStoreAVL implements PersistentStore {
 
     public void setMemory(boolean mode) {}
 
-    public abstract int getAccessCount();
-
     public abstract void set(CachedObject object);
 
     public abstract CachedObject get(long key, boolean keep);
@@ -202,23 +206,27 @@ public abstract class RowStoreAVL implements PersistentStore {
      */
     public void delete(Session session, Row row) {
 
-        for (int i = 0; i < indexList.length; i++) {
-            indexList[i].delete(session, this, row);
-        }
+        writeLock();
 
-        for (int i = 0; i < subStores.length; i++) {
-            subStores[i].delete(session, row);
-        }
+        try {
+            for (int i = 0; i < indexList.length; i++) {
+                indexList[i].delete(session, this, row);
+            }
 
-        row.delete(this);
+            for (int i = 0; i < subStores.length; i++) {
+                subStores[i].delete(session, row);
+            }
 
-        long count = elementCount.decrementAndGet();
+            row.delete(this);
 
-        if (count > 16 * 1024 && count < baseElementCount / 2) {
-            synchronized (this) {
+            long count = elementCount.decrementAndGet();
+
+            if (count > 16 * 1024 && count < baseElementCount / 2) {
                 baseElementCount = count;
                 searchCost       = null;
             }
+        } finally {
+            writeUnlock();
         }
     }
 
@@ -226,6 +234,8 @@ public abstract class RowStoreAVL implements PersistentStore {
 
         int i = 0;
 
+        writeLock();
+
         try {
             for (; i < indexList.length; i++) {
                 indexList[i].insert(session, this, row);
@@ -254,10 +264,8 @@ public abstract class RowStoreAVL implements PersistentStore {
             long count = elementCount.incrementAndGet();
 
             if (count > 16 * 1024 && count > baseElementCount * 2) {
-                synchronized (this) {
-                    baseElementCount = count;
-                    searchCost       = null;
-                }
+                baseElementCount = count;
+                searchCost       = null;
             }
         } catch (HsqlException e) {
             int count = i;
@@ -284,26 +292,34 @@ public abstract class RowStoreAVL implements PersistentStore {
 
             // do not remove as there may be still be reference
             throw Error.error(ErrorCode.GENERAL_ERROR, t);
+        } finally {
+            writeUnlock();
         }
     }
 
     //
     public final void indexRows(Session session) {
 
-        for (int i = 1; i < indexList.length; i++) {
-            setAccessor(indexList[i], null);
-        }
-
-        RowIterator it = rowIterator();
-
-        while (it.hasNext()) {
-            Row row = it.getNextRow();
-
-            ((RowAVL) row).clearNonPrimaryNodes();
+        writeLock();
 
+        try {
             for (int i = 1; i < indexList.length; i++) {
-                indexList[i].insert(session, this, row);
+                setAccessor(indexList[i], null);
             }
+
+            RowIterator it = rowIterator();
+
+            while (it.hasNext()) {
+                Row row = it.getNextRow();
+
+                ((RowAVL) row).clearNonPrimaryNodes();
+
+                for (int i = 1; i < indexList.length; i++) {
+                    indexList[i].insert(session, this, row);
+                }
+            }
+        } finally {
+            writeUnlock();
         }
     }
 
@@ -323,10 +339,7 @@ public abstract class RowStoreAVL implements PersistentStore {
     }
 
     public void setAccessor(Index key, CachedObject accessor) {
-
-        Index index = (Index) key;
-
-        accessorList[index.getPosition()] = accessor;
+        accessorList[key.getPosition()] = accessor;
     }
 
     public void setAccessor(Index key, long accessor) {}
@@ -443,7 +456,13 @@ public abstract class RowStoreAVL implements PersistentStore {
         Index index = this.indexList[0];
 
         if (elementCount.get() < 0) {
-            elementCount.set(index.getNodeCount(null, this));
+            readLock();
+
+            try {
+                elementCount.set(index.getNodeCount(null, this));
+            } finally {
+                readUnlock();
+            }
         }
 
         return elementCount.get();
@@ -451,29 +470,28 @@ public abstract class RowStoreAVL implements PersistentStore {
 
     public long elementCount(Session session) {
 
-        Index index = this.indexList[0];
-
-        if (elementCount.get() < 0) {
-            elementCount.set(index.getNodeCount(session, this));
-        }
-
         if (session != null) {
-            int txControl = session.database.txManager.getTransactionControl();
+            Index index = this.indexList[0];
 
-            if (txControl != TransactionManager.LOCKS) {
+            if (session.database.txManager.isMVRows()) {
                 switch (table.getTableType()) {
 
                     case TableBase.MEMORY_TABLE :
                     case TableBase.CACHED_TABLE :
                     case TableBase.TEXT_TABLE :
-                        return index.getNodeCount(session, this);
+                        readLock();
 
+                        try {
+                            return index.getNodeCount(session, this);
+                        } finally {
+                            readUnlock();
+                        }
                     default :
                 }
             }
         }
 
-        return elementCount.get();
+        return elementCount();
     }
 
     public long elementCountUnique(Index index) {
@@ -506,11 +524,11 @@ public abstract class RowStoreAVL implements PersistentStore {
             ColumnSchema column = ((Table) table).getColumn(colindex);
 
             colvalue = column.getDefaultValue(session);
-            newtype  = ((Table) table).getColumnTypes()[colindex];
+            newtype  = table.getColumnTypes()[colindex];
         }
 
         if (adjust <= 0 && colindex != -1) {
-            oldtype = ((Table) other.getTable()).getColumnTypes()[colindex];
+            oldtype = other.getTable().getColumnTypes()[colindex];
         }
 
         try {
@@ -584,15 +602,21 @@ public abstract class RowStoreAVL implements PersistentStore {
 
     public void reindex(Session session, Index index) {
 
-        setAccessor(index, null);
+        writeLock();
 
-        RowIterator it = table.rowIterator(this);
+        try {
+            setAccessor(index, null);
 
-        while (it.hasNext()) {
-            RowAVL row = (RowAVL) it.getNextRow();
+            RowIterator it = table.rowIterator(this);
 
-            row.getNode(index.getPosition()).delete();
-            index.insert(session, this, row);
+            while (it.hasNext()) {
+                RowAVL row = (RowAVL) it.getNextRow();
+
+                row.getNode(index.getPosition()).delete();
+                index.insert(session, this, row);
+            }
+        } finally {
+            writeUnlock();
         }
     }
 
@@ -629,51 +653,57 @@ public abstract class RowStoreAVL implements PersistentStore {
     boolean insertIndexNodes(Session session, Index primaryIndex,
                              Index newIndex) {
 
-        int           position = newIndex.getPosition();
-        RowIterator   it       = primaryIndex.firstRow(this);
-        int           rowCount = 0;
-        HsqlException error    = null;
+        writeLock();
 
         try {
-            while (it.hasNext()) {
-                Row row = it.getNextRow();
+            int           position = newIndex.getPosition();
+            RowIterator   it       = primaryIndex.firstRow(this);
+            int           rowCount = 0;
+            HsqlException error    = null;
 
-                ((RowAVL) row).insertNode(position);
+            try {
+                while (it.hasNext()) {
+                    Row row = it.getNextRow();
 
-                // count before inserting
-                rowCount++;
+                    ((RowAVL) row).insertNode(position);
 
-                newIndex.insert(session, this, row);
+                    // count before inserting
+                    rowCount++;
+
+                    newIndex.insert(session, this, row);
+                }
+
+                it.release();
+
+                return true;
+            } catch (OutOfMemoryError e) {
+                error = Error.error(ErrorCode.OUT_OF_MEMORY);
+            } catch (HsqlException e) {
+                error = e;
+            }
+
+            // backtrack on error
+            // rowCount rows have been modified
+            it = primaryIndex.firstRow(this);
+
+            for (int i = 0; i < rowCount; i++) {
+                Row     row      = it.getNextRow();
+                NodeAVL backnode = ((RowAVL) row).getNode(0);
+                int     j        = position;
+
+                while (--j > 0) {
+                    backnode = backnode.nNext;
+                }
+
+                backnode.nNext = backnode.nNext.nNext;
             }
 
             it.release();
 
-            return true;
-        } catch (OutOfMemoryError e) {
-            error = Error.error(ErrorCode.OUT_OF_MEMORY);
-        } catch (HsqlException e) {
-            error = e;
+            throw error;
+        } finally {
+            writeUnlock();
         }
-
-        // backtrack on error
-        // rowCount rows have been modified
-        it = primaryIndex.firstRow(this);
-
-        for (int i = 0; i < rowCount; i++) {
-            Row     row      = it.getNextRow();
-            NodeAVL backnode = ((RowAVL) row).getNode(0);
-            int     j        = position;
-
-            while (--j > 0) {
-                backnode = backnode.nNext;
-            }
-
-            backnode.nNext = backnode.nNext.nNext;
-        }
-
-        it.release();
-
-        throw error;
     }
 
     /**
