@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2011, The HSQL Development Group
+/* Copyright (c) 2001-2015, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@ public class RowStoreDataChange extends RowStoreAVLHybrid {
                               PersistentStoreCollection manager,
                               TableBase table) {
 
-        super(session, manager, table, true);
+        super(session, table, true);
 
         this.session = session;
 
