@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,16 +65,17 @@ import org.hsqldb.types.Type;
  *  query session settings, to allocate and execute statements, etc.) and all
  *  responses (such as exception indications, update counts, result sets and
  *  result set metadata). It also implements the HSQL wire protocol for
- *  comunicating all such requests and responses across the network.
+ *  communicating all such requests and responses across the network.
  *  Uses a navigator for data.
  *
  * @author Campbell Burnet (boucherb@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.9.0
  */
 public class Result {
 
+    public static final Result[]       emptyArray = new Result[0];
     public static final ResultMetaData sessionAttributesMetaData;
 
     static {
@@ -244,15 +245,13 @@ public class Result {
     }
 
     public static Result newResult(DataInput dataInput,
-                                   RowInputBinary in)
-                                   throws IOException {
+                                   RowInputBinary in) throws IOException {
         return newResult(null, dataInput.readByte(), dataInput, in);
     }
 
     public static Result newResult(Session session, int mode,
                                    DataInput dataInput,
-                                   RowInputBinary in)
-                                   throws IOException {
+                                   RowInputBinary in) throws IOException {
 
         try {
             if (mode == ResultConstants.LARGE_OBJECT_OP) {
@@ -269,8 +268,7 @@ public class Result {
 
     public void readAdditionalResults(SessionInterface session,
                                       DataInputStream inputStream,
-                                      RowInputBinary in)
-                                      throws IOException {
+                                      RowInputBinary in) throws IOException {
 
         Result currentResult = this;
 
@@ -291,8 +289,7 @@ public class Result {
 
     public void readLobResults(SessionInterface session,
                                DataInputStream inputStream,
-                               RowInputBinary in)
-                               throws IOException {
+                               RowInputBinary in) throws IOException {
 
         Result  currentResult = this;
         boolean hasLob        = false;
@@ -329,8 +326,7 @@ public class Result {
 
     private static Result newResult(Session session, DataInput dataInput,
                                     RowInputBinary in,
-                                    int mode)
-                                    throws IOException {
+                                    int mode) throws IOException {
 
         Result result = newResult(mode);
         int    length = dataInput.readInt();
@@ -413,6 +409,7 @@ public class Result {
                 result.sessionID    = in.readLong();
                 result.databaseName = in.readString();
                 result.mainString   = in.readString();
+                result.generateKeys = in.readInt();
                 break;
 
             case ResultConstants.UPDATECOUNT :
@@ -464,6 +461,14 @@ public class Result {
 
                 break;
             }
+            case ResultConstants.SQLCANCEL :
+                result.databaseID   = in.readInt();
+                result.sessionID    = in.readLong();
+                result.statementID  = in.readLong();
+                result.generateKeys = in.readInt();
+                result.mainString   = in.readString();
+                break;
+
             case ResultConstants.PREPARE_ACK :
                 result.statementReturnType = in.readByte();
                 result.statementID         = in.readLong();
@@ -604,7 +609,7 @@ public class Result {
 
     /**
      * For SQLPREPARE
-     * For parparation of SQL parepared statements.
+     * For preparation of SQL prepared statements.
      */
     public static Result newPrepareStatementRequest() {
         return newResult(ResultConstants.PREPARE);
@@ -699,7 +704,7 @@ public class Result {
     }
 
     public void addBatchedPreparedExecuteRequest(Object[] parameterValues) {
-        ((RowSetNavigatorClient) navigator).add(parameterValues);
+        navigator.add(parameterValues);
     }
 
     /**
@@ -763,16 +768,17 @@ public class Result {
         return result;
     }
 
-    public static Result newConnectionAcknowledgeResponse(Database database,
-            long sessionID, int databaseID) {
+    public static Result newConnectionAcknowledgeResponse(Session session) {
 
         Result result = newResult(ResultConstants.CONNECTACKNOWLEDGE);
 
-        result.sessionID    = sessionID;
-        result.databaseID   = databaseID;
-        result.databaseName = database.getUniqueName();
+        result.sessionID    = session.getId();
+        result.databaseID   = session.getDatabase().getDatabaseID();
+        result.databaseName = session.getDatabase().getNameString();
         result.mainString =
-            database.getProperties().getClientPropertiesAsString();
+            session.getDatabase().getProperties()
+                .getClientPropertiesAsString();
+        result.generateKeys = session.getRandomId();
 
         return result;
     }
@@ -857,6 +863,16 @@ public class Result {
         return r;
     }
 
+    public static Result newCancelRequest(long statementId, String sql) {
+
+        Result r = newResult(ResultConstants.SQLCANCEL);
+
+        r.statementID = statementId;
+        r.mainString  = sql;
+
+        return r;
+    }
+
     public static Result newFreeStmtRequest(long statementID) {
 
         Result r = newResult(ResultConstants.FREESTMT);
@@ -1049,24 +1065,23 @@ public class Result {
             result.mainString = result.exception.getMessage();
             result.subString  = result.exception.getSQLState();
             result.errorCode  = result.exception.getErrorCode();
+        } else if (t instanceof Throwable) {
+            result.exception  = Error.error(ErrorCode.GENERAL_ERROR, t);
+            result.mainString = result.exception.getMessage();
+            result.subString  = result.exception.getSQLState();
+            result.errorCode  = result.exception.getErrorCode();
         } else {
-            result.exception = Error.error(ErrorCode.GENERAL_ERROR, t);
-            result.mainString = result.exception.getMessage() + " "
-                                + t.toString();
-            result.subString = result.exception.getSQLState();
-            result.errorCode = result.exception.getErrorCode();
-
-            if (statement != null) {
-                result.mainString += " in statement [" + statement + "]";
-            }
+            result.exception  = Error.error(ErrorCode.GENERAL_ERROR);
+            result.mainString = result.exception.getMessage();
+            result.subString  = result.exception.getSQLState();
+            result.errorCode  = result.exception.getErrorCode();
         }
 
         return result;
     }
 
     public void write(SessionInterface session, DataOutputStream dataOut,
-                      RowOutputInterface rowOut)
-                      throws IOException {
+                      RowOutputInterface rowOut) throws IOException {
 
         rowOut.reset();
         rowOut.writeByte(mode);
@@ -1143,6 +1158,7 @@ public class Result {
                 rowOut.writeLong(sessionID);
                 rowOut.writeString(databaseName);
                 rowOut.writeString(mainString);
+                rowOut.writeInt(generateKeys);
                 break;
 
             case ResultConstants.UPDATECOUNT :
@@ -1174,6 +1190,14 @@ public class Result {
 
                 break;
             }
+            case ResultConstants.SQLCANCEL :
+                rowOut.writeInt(databaseID);
+                rowOut.writeLong(sessionID);
+                rowOut.writeLong(statementID);
+                rowOut.writeInt(generateKeys);
+                rowOut.writeString(mainString);
+                break;
+
             case ResultConstants.PREPARE_ACK :
                 rowOut.writeByte(statementReturnType);
                 rowOut.writeLong(statementID);
@@ -1459,7 +1483,7 @@ public class Result {
 
     public Object[] getSingleRowData() {
 
-        Object[] data = (Object[]) initialiseNavigator().getNext();
+        Object[] data = initialiseNavigator().getNext();
 
         data = (Object[]) ArrayUtil.resizeArrayIfDifferent(data,
                 metaData.getColumnCount());
@@ -1472,7 +1496,7 @@ public class Result {
     }
 
     public Object[] getSessionAttributes() {
-        return (Object[]) initialiseNavigator().getNext();
+        return initialiseNavigator().getNext();
     }
 
     public void setResultType(int type) {
@@ -1487,6 +1511,14 @@ public class Result {
         return statementReturnType;
     }
 
+    public void setSessionRandomID(int id) {
+        generateKeys = id;
+    }
+
+    public int getSessionRandomID() {
+        return generateKeys;
+    }
+
     public int getGeneratedResultType() {
         return generateKeys;
     }
@@ -1554,6 +1586,30 @@ public class Result {
         lobCount   = 0;
     }
 
+    public void addRows(String[] sql) {
+
+        if (sql == null) {
+            return;
+        }
+
+        for (int i = 0; i < sql.length; i++) {
+            String[] s = new String[1];
+
+            s[0] = sql[i];
+
+            initialiseNavigator().add(s);
+        }
+    }
+
+    public void addRow(String[] sql) {
+
+        if (sql == null) {
+            return;
+        }
+
+        initialiseNavigator().add(sql);
+    }
+
     private static Object[] readSimple(RowInputBinary in,
                                        ResultMetaData meta)
                                        throws IOException {
