@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,7 @@ import org.hsqldb.Tokens;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.map.ValuePool;
+import org.hsqldb.persist.TextFileSettings;
 import org.hsqldb.types.BinaryData;
 import org.hsqldb.types.BlobData;
 import org.hsqldb.types.BlobDataID;
@@ -56,28 +57,28 @@ import org.hsqldb.types.Types;
  * Class for reading the data for a database row in text table format.
  *
  * @author Bob Preston (sqlbob@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.7.0
  */
 public class RowInputText extends RowInputBase implements RowInputInterface {
 
     // text table specific
-    private String    fieldSep;
-    private String    varSep;
-    private String    longvarSep;
-    private int       fieldSepLen;
-    private int       varSepLen;
-    private int       longvarSepLen;
-    private boolean   fieldSepEnd;
-    private boolean   varSepEnd;
-    private boolean   longvarSepEnd;
-    private int       textLen;
-    protected String  text;
-    protected int     line;
-    protected int     field;
-    protected int     next = 0;
-    protected boolean allQuoted;
-    protected Scanner scanner;
+    protected TextFileSettings textFileSettings;
+    private String             fieldSep;
+    private String             varSep;
+    private String             longvarSep;
+    private int                fieldSepLen;
+    private int                varSepLen;
+    private int                longvarSepLen;
+    private boolean            fieldSepEnd;
+    private boolean            varSepEnd;
+    private boolean            longvarSepEnd;
+    private int                textLen;
+    protected String           text;
+    protected long             line;
+    protected int              field;
+    protected int              next = 0;
+    protected Scanner          scanner;
 
     //
     private int maxPooledStringLength = ValuePool.getMaxStringLength();
@@ -86,12 +87,15 @@ public class RowInputText extends RowInputBase implements RowInputInterface {
      * fredt@users - comment - in future may use a custom subclasse of
      * InputStream to read the data.
      */
-    public RowInputText(String fieldSep, String varSep, String longvarSep,
-                        boolean allQuoted) {
+    public RowInputText(TextFileSettings textFileSettings) {
 
         super(new byte[0]);
 
-        scanner = new Scanner();
+        scanner               = new Scanner();
+        this.textFileSettings = textFileSettings;
+        this.fieldSep         = textFileSettings.fs;
+        this.varSep           = textFileSettings.vs;
+        this.longvarSep       = textFileSettings.lvs;
 
         //-- Newline indicates that field should match to end of line.
         if (fieldSep.endsWith("\n")) {
@@ -109,13 +113,9 @@ public class RowInputText extends RowInputBase implements RowInputInterface {
             longvarSep    = longvarSep.substring(0, longvarSep.length() - 1);
         }
 
-        this.allQuoted  = allQuoted;
-        this.fieldSep   = fieldSep;
-        this.varSep     = varSep;
-        this.longvarSep = longvarSep;
-        fieldSepLen     = fieldSep.length();
-        varSepLen       = varSep.length();
-        longvarSepLen   = longvarSep.length();
+        fieldSepLen   = fieldSep.length();
+        varSepLen     = varSep.length();
+        longvarSepLen = longvarSep.length();
     }
 
     public void setSource(String text, long pos, int byteSize) {
@@ -259,7 +259,7 @@ public class RowInputText extends RowInputBase implements RowInputInterface {
         }
 
         if (s.length() > this.maxPooledStringLength) {
-            return new String(s);
+            return s;
         } else {
             return ValuePool.getString(s);
         }
@@ -539,7 +539,7 @@ public class RowInputText extends RowInputBase implements RowInputInterface {
         throw Error.runtimeError(ErrorCode.U_S0500, "RowInputText");
     }
 
-    public int getLineNumber() {
+    public long getLineNumber() {
         return line;
     }
 
