@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,9 +36,7 @@ import java.net.ServerSocket;
 import java.net.Socket;
 import java.net.UnknownHostException;
 import java.security.Principal;
-import java.security.Provider;
 import java.security.PublicKey;
-import java.security.Security;
 
 import javax.net.ssl.HandshakeCompletedEvent;
 import javax.net.ssl.HandshakeCompletedListener;
@@ -74,7 +72,7 @@ implements HandshakeCompletedListener {
     protected Object serverSocketFactory;
 
     /**
-     * Monitor object to guard against conncurrent modification
+     * Monitor object to guard against concurrent modification
      * of the underlying socket factory implementation member.
      */
     protected final Object socket_factory_mutex = new Object();
@@ -88,7 +86,7 @@ implements HandshakeCompletedListener {
 // ------------------------------ constructors ---------------------------------
 
     /**
-     * External construction disabled.  New factory instances are retreived
+     * External construction disabled.  New factory instances are retrieved
      * through the newHsqlSocketFactory method instead.
      */
     protected HsqlSocketFactorySecure() throws Exception {
@@ -177,7 +175,7 @@ implements HandshakeCompletedListener {
     }
 
     /**
-     * if socket argurment is not null, creates a secure Socket as a wapper for
+     * if socket argument is not null, creates a secure Socket as a wrapper for
      * the normal, non-SSL socket. If the socket is null, create a new secure
      * socket. The secure socket is configured using the
      * socket options established for this factory.
@@ -243,13 +241,13 @@ implements HandshakeCompletedListener {
 //
 // Reference:  http://www.securitytracker.com/alerts/2002/Aug/1005030.html
 //
-// That is, we can't guarantee that installed/prefered provider trust manager
+// That is, we can't guarantee that installed/preferred provider trust manager
 // implementations verify the whole chain properly and there are still
 // v1 certs out there (i.e. have no basic constraints, etc.), meaning that
 // we should check for and reject any intermediate certs that are not v3+
 // (cannot be checked for basic constraints).  Only root and intermediate
 // certs found in the trust store should be allowed to be v1 (since we must
-// be trusing them for them to be there).  All other intermediate signers,
+// be trusting them for them to be there).  All other intermediate signers,
 // however, should be required to be v3+, otherwise anybody with any kind
 // of cert issued somehow via a trust chain from the root can pose as an
 // intermediate signing CA and hence leave things open to man-in-the-middle
@@ -325,7 +323,7 @@ implements HandshakeCompletedListener {
     }
 
     /**
-     * Verifyies the certificate chain presented by the server to which
+     * Verifies the certificate chain presented by the server to which
      * a secure Socket has just connected.  Specifically, the provided host
      * name is checked against the Common Name of the server certificate;
      * additional checks may or may not be performed.
